#include "basefun.h"

/**
   function computes linear base functions on bar element
   natural coordinate is from segment <-1;1>
   
   @param n - array of base functions
   @param x - natural coodinate
*/
void bf_lin_1d (double *n,double x)
{
  n[0]=0.5*(1.0-x);
  n[1]=0.5*(1.0+x);
}

/**
   function computes derivatives of linear base functions on bar element with respect of natural coordinate x
   
   @param n - array of derivatives of base functions
*/
void dx_bf_lin_1d (double *n)
{
  n[0]=-0.5;
  n[1]= 0.5;
}

/**
   function computes quadratic approximation functions on bar element
   
   @param n - array containing approximation functions
   @param x - natural coordinate
*/
void bf_quad_1d (double *n,double x)
{
  n[0]=x*(x-1.0)/2.0;
  n[1]=x*(1.0+x)/2.0;
  n[2]=1.0-x*x;
}

/**
   function computes derivatives of quadratic approximation functions 
   with respect of x on bar element
   
   @param n - array containing approximation functions
   @param x - natural coordinate
*/
void dx_bf_quad_1d (double *n,double x)
{
  n[0]=x-0.5;
  n[1]=x+0.5;
  n[2]=-2.0*x;
}

/**
   function computes approximation function of dilatation of 2D beam element
   
   @param n - array containing approximation functions
   @param xi - natural coordinate from segment <0;1>
   
   20.2.2002
*/
void dilat (double *n,double xi)
{
  n[0] = 1.0-xi;
  n[1] = xi;
}

/**
   function computes derivative of approximation function of dilatation of 2D beam element with respect of x
   
   @param n - array containing approximation functions
   @param l - length of the beam
   
   20.2.2002
*/
void der_dilat (double *n,double l)
{
  n[0] = -1.0/l;
  n[1] = 1.0/l;
}

/**
   function computes approximation function of deflection of 2D beam element
   
   @param n - array containing approximation functions
   @param xi - natural coordinate from segment <0;1>
   @param l - length of the beam
   @param kappa - 6.0*E*I/k/G/A/l/l
   
   20.2.2002
*/
void defl2D_fun (double *n,double xi,double l,double kappa)
{
  n[0] = (2.0*xi*xi*xi - 3.0*xi*xi - 2.0*kappa*xi + 2.0*kappa+1.0)/(1.0+2.0*kappa);
  n[1] = (-1.0*xi*xi*xi + (2.0+kappa)*xi*xi - (kappa+1.0)*xi)*l/(1.0+2.0*kappa);
  n[2] = (-2.0*xi*xi*xi + 3.0*xi*xi + 2.0*kappa*xi)/(1.0+2.0*kappa);
  n[3] = (-1.0*xi*xi*xi + (1.0-kappa)*xi*xi + kappa*xi)*l/(1.0+2.0*kappa);
}

/**
   function computes derivative of deflection function with respect of x

   @param n - array containing approximation functions
   @param xi - natural coordinate from segment <0;1>
   @param l - length of the beam
   @param kappa - 6.0*E*I/k/G/A/l/l
   
   20.2.2002
*/
void der_defl2D_fun (double *n,double xi,double l,double kappa)
{
  n[0] = (6.0*xi*xi - 6.0*xi - 2.0*kappa)/l/(1.0+2.0*kappa);
  n[1] = (-3.0*xi*xi + 2.0*(2.0+kappa)*xi - (kappa+1.0))/(1.0+2.0*kappa);
  n[2] = (-6.0*xi*xi + 6.0*xi + 2.0*kappa)/l/(1.0+2.0*kappa);
  n[3] = (-3.0*xi*xi + 2.0*(1.0-kappa)*xi + kappa)/(1.0+2.0*kappa);
}

/**
   function computes approximation function of rotation of 2D beam element
   
   @param n - array containing approximation functions
   @param xi - natural coordinate from segment <0;1>
   @param l - length of the beam
   @param kappa - 6.0*E*I/k/G/A/l/l
   
   20.2.2002
*/
void roty_fun (double *n,double xi,double l,double kappa)
{
  n[0] = (-6.0*xi*xi + 6.0*xi)/l/(1.0+2.0*kappa);
  n[1] = (3.0*xi*xi - 2.0*(kappa+2.0)*xi + 1.0+2.0*kappa)/(1.0+2.0*kappa);
  n[2] = (6.0*xi*xi - 6.0*xi)/l/(1.0+2.0*kappa);
  n[3] = (3.0*xi*xi - 2.0*(1.0-kappa)*xi)/(1.0+2.0*kappa);
}

/**
   function computes derivative of function of rotation of 2D beam element with respect of x
   
   @param n - array containing approximation functions
   @param xi - natural coordinate from segment <0;1>
   @param l - length of the beam
   @param kappa - 6.0*E*I/k/G/A/l/l
   
   20.2.2002
*/
void der_roty_fun (double *n,double xi,double l,double kappa)
{
  n[0] = (-12.0*xi + 6.0)/l/l/(1.0+2.0*kappa);
  n[1] = (6.0*xi - 2.0*(kappa+2.0))/l/(1.0+2.0*kappa);
  n[2] = (12.0*xi - 6.0)/l/l/(1.0+2.0*kappa);
  n[3] = (6.0*xi - 2.0*(1.0-kappa))/l/(1.0+2.0*kappa);
}


/**
   function computes a_i coefficients of area-coordinates on triangles
   L(x,y)_i = (a_i + b_i.x + c_i.y)/area/2
   
   @param a - coefficients
   @param x,y - node coordinates
*/
void a_coeff (double *a,double *x,double *y)
{
  a[0]=x[1]*y[2]-x[2]*y[1];
  a[1]=x[2]*y[0]-x[0]*y[2];
  a[2]=x[0]*y[1]-x[1]*y[0];
}



/**
   function computes b_i coefficients of area-coordinates on triangles
   L(x,y)_i = (a_i + b_i.x + c_i.y)/area/2
   
   @param b - coefficients
   @param y - node coordinates
*/
void b_coeff (double *b,double *y)
{
  b[0]=y[1]-y[2];
  b[1]=y[2]-y[0];
  b[2]=y[0]-y[1];
}



/**
   function computes c_i coefficients of area-coordinates on triangles
   L(x,y)_i = (a_i + b_i.x + c_i.y)/area/2
   
   @param c - coefficients
   @param x - node coordinates
*/
void c_coeff (double *c,double *x)
{
  c[0]=x[2]-x[1];
  c[1]=x[0]-x[2];
  c[2]=x[1]-x[0];
}



/**
   function computes a_i coefficients of area-coordinates on triangles
   L(x,y)_i = a_i + b_i.x + c_i.y
   
   @param a - coefficients
   @param x,y - node coordinates
   @param det - determinant=2*area
*/
void plsa (double *a,double *x,double *y,double det)
{
  a[0]=(x[1]*y[2]-x[2]*y[1])/det;
  a[1]=(x[2]*y[0]-x[0]*y[2])/det;
  a[2]=(x[0]*y[1]-x[1]*y[0])/det;
}



/**
   function computes b_i coefficients of area-coordinates on triangles
   L(x,y)_i = a_i + b_i.x + c_i.y
   
   @param b - coefficients
   @param y - node coordinates
   @param det - determinant=2*area
*/
void plsb (double *b,double *y,double det)
{
  b[0]=(y[1]-y[2])/det;
  b[1]=(y[2]-y[0])/det;
  b[2]=(y[0]-y[1])/det;
}



/**
   function computes c_i coefficients of area-coordinates on triangles
   L(x,y)_i = a_i + b_i.x + c_i.y
   
   @param c - coefficients
   @param x - node coordinates
   @param det - determinant=2*area
*/
void plsc (double *c,double *x,double det)
{
  c[0]=(x[2]-x[1])/det;
  c[1]=(x[0]-x[2])/det;
  c[2]=(x[1]-x[0])/det;
}



/**
   function computes quadratic base functions on triangle
   
   @param n - array of base functions
   @param l - areacoordinates
*/
void ac_bf_quad_3_2d (double *n,double *l)
{
  n[0]=l[0]*(2.0*l[0]-1.0);
  n[1]=l[1]*(2.0*l[1]-1.0);
  n[2]=l[2]*(2.0*l[2]-1.0);

  n[3]=l[0]*l[1]*4.0;
  n[4]=l[1]*l[2]*4.0;
  n[5]=l[2]*l[0]*4.0;

  //n[3]=l[1]*l[2]*4.0;
  //n[4]=l[2]*l[0]*4.0;
  //n[5]=l[0]*l[1]*4.0;
}



/**
   function computes derivatives of quadratic base functions on triangle with respect of x
   
   @param n - array of derivatives of base functions
   @param b - array of coefficients of area coordinates
   @param l - areacoordinates
*/
void ac_dx_bf_quad_3_2d (double *n,double *b,double *l)
{
  n[0]=b[0]*(2.0*l[0]-1.0)+l[0]*2.0*b[0];
  n[1]=b[1]*(2.0*l[1]-1.0)+l[1]*2.0*b[1];
  n[2]=b[2]*(2.0*l[2]-1.0)+l[2]*2.0*b[2];

  n[3]=(b[0]*l[1]+l[0]*b[1])*4.0;
  n[4]=(b[1]*l[2]+l[1]*b[2])*4.0;
  n[5]=(b[2]*l[0]+l[2]*b[0])*4.0;

  //n[3]=(b[1]*l[2]+l[1]*b[2])*4.0;
  //n[4]=(b[2]*l[0]+l[2]*b[0])*4.0;
  //n[5]=(b[0]*l[1]+l[0]*b[1])*4.0;
}



/**
   function computes derivatives of quadratic base functions on triangle with respect of y
   
   @param n - array of derivatives of base functions
   @param c - array of coefficients of area coordinates
   @param l - areacoordinates
*/
void ac_dy_bf_quad_3_2d (double *n,double *c,double *l)
{
  n[0]=c[0]*(2.0*l[0]-1.0)+l[0]*2.0*c[0];
  n[1]=c[1]*(2.0*l[1]-1.0)+l[1]*2.0*c[1];
  n[2]=c[2]*(2.0*l[2]-1.0)+l[2]*2.0*c[2];

  n[3]=(c[0]*l[1]+l[0]*c[1])*4.0;
  n[4]=(c[1]*l[2]+l[1]*c[2])*4.0;
  n[5]=(c[2]*l[0]+l[2]*c[0])*4.0;

  //n[3]=(c[1]*l[2]+l[1]*c[2])*4.0;
  //n[4]=(c[2]*l[0]+l[2]*c[0])*4.0;
  //n[5]=(c[0]*l[1]+l[0]*c[1])*4.0;
}


/**
   function computes linear approximation functions on triangle
   
   @param n - array containing approximation functions
   @param x,y - natural coordinates
   
   1.4.2002
*/
void bf_lin_3_2d (double *n,double x,double y)
{
  n[0]=x;
  n[1]=y;
  n[2]=1.0-x-y;
}

/**
   function computes derivatives of linear approximation functions with respect to natural coordinate xi
   
   @param n - array containing derivatives
   
   1.4.2002
*/
void dx_bf_lin_3_2d (double *n)
{
  n[0] =  1.0;
  n[1] =  0.0;
  n[2] = -1.0;
}

/**
   function computes derivatives of linear approximation functions with respect to natural coordinate eta
   
   @param n - array containing derivatives
   
   1.4.2002
*/
void dy_bf_lin_3_2d (double *n)
{
  n[0] =  0.0;
  n[1] =  1.0;
  n[2] = -1.0;
}

/**
   function computes quadratic approximation functions on triangle
   
   @param n - array containing approximation functions
   @param x,y - natural coordinates
   
   1.4.2002
*/
void bf_quad_3_2d (double *n,double x,double y)
{
  n[0]=x*(x-0.5)*2.0;
  n[1]=y*(y-0.5)*2.0;
  n[2]=(1.0-x-y)*(0.5-x-y)*2.0;
  n[3]=x*y*4.0;
  n[4]=y*(1.0-x-y)*4.0;
  n[5]=x*(1.0-x-y)*4.0;
}

/**
   function computes derivatives of quadratic approximation functions on triangle with respect to xi
   
   @param n - array containing approximation functions
   @param x,y - natural coordinates
   
   1.4.2002
*/
void dx_bf_quad_3_2d (double *n,double x,double y)
{
  n[0]=4.0*x-1.0;
  n[1]=0.0;
  n[2]=4.0*x+4.0*y-3.0;
  n[3]=4.0*y;
  n[4]=-4.0*y;
  n[5]=4.0-8.0*x-4.0*y;
}

/**
   function computes derivatives of quadratic approximation functions on triangle with respect to eta
   
   @param n - array containing approximation functions
   @param x,y - natural coordinates
   
   1.4.2002
*/
void dy_bf_quad_3_2d (double *n,double x,double y)
{
  n[0]=0.0;
  n[1]=4.0*y-1.0;
  n[2]=4.0*x+4.0*y-3.0;
  n[3]=4.0*x;
  n[4]=4.0-4.0*x-8.0*y;
  n[5]=-4.0*x;
}



/**
   function computes bi-linear base functions on quadrilateral
   
   @param n - array of base functions
   @param x,y - natural coodinates
*/
void bf_lin_4_2d (double *n,double x,double y)
{
  n[0]=0.25*(1.0+x)*(1.0+y);
  n[1]=0.25*(1.0-x)*(1.0+y);
  n[2]=0.25*(1.0-x)*(1.0-y);
  n[3]=0.25*(1.0+x)*(1.0-y);
}



/**
   function computes derivatives of bi-linear base functions on quadrilateral with respect of natural coordinate x
   
   @param n - array of derivatives of base functions
   @param y - natural coodinate
*/
void dx_bf_lin_4_2d (double *n,double y)
{
  n[0]=0.25*(1.0+y);
  n[1]=0.25*(-1.0-y);
  n[2]=0.25*(-1.0+y);
  n[3]=0.25*(1.0-y);
}



/**
   function computes derivatives of bi-linear base functions on quadrilateral with respect of natural coordinate y
   
   @param n - array of derivatives of base functions
   @param x - natural coodinate
*/
void dy_bf_lin_4_2d (double *n,double x)
{
  n[0]=0.25*(1.0+x);
  n[1]=0.25*(1.0-x);
  n[2]=0.25*(-1.0+x);
  n[3]=0.25*(-1.0-x);
}



/**
   function computes bi-quadratic base functions on quadrilateral
   
   @param n - array of base functions
   @param x,y - natural coodinates
*/
void bf_quad_4_2d (double *n,double x,double y)
{
  n[0]=0.25*(1.0+x)*(1.0+y)*(x+y-1.0);
  n[1]=0.25*(1.0-x)*(1.0+y)*(y-x-1.0);
  n[2]=0.25*(1.0-x)*(1.0-y)*(-x-y-1.0);
  n[3]=0.25*(1.0+x)*(1.0-y)*(x-y-1.0);
  n[4]=0.5*(1.0-x*x)*(1.0+y);
  n[5]=0.5*(1.0-x)*(1.0-y*y);
  n[6]=0.5*(1.0-x*x)*(1.0-y);
  n[7]=0.5*(1.0+x)*(1.0-y*y);
}



/**
   function computes derivatives of bi-quadratic base functions on quadrilateral with respect of natural coordinate x
   
   @param n - array of derivatives of base functions
   @param x,y - natural coodinates
*/
void dx_bf_quad_4_2d (double *n,double x,double y)
{
  n[0]=0.25*(1.0+y)*(x+y-1.0)+0.25*(1.0+x)*(1.0+y);
  n[1]=0.25*(-1.0-y)*(y-x-1.0)-0.25*(1.0-x)*(1.0+y);
  n[2]=0.25*(-1.0+y)*(-x-y-1.0)-0.25*(1.0-x)*(1.0-y);
  n[3]=0.25*(1.0-y)*(x-y-1.0)+0.25*(1.0+x)*(1.0-y);
  n[4]=(1.0+y)*(-1.0)*x;
  n[5]=0.5*(y*y-1.0);
  n[6]=(y-1)*x;
  n[7]=0.5*(1.0-y*y);
}



/**
   function computes derivatives of bi-quadratic base functions on quadrilateral with respect of natural coordinate y
   
   @param n - array of derivatives of base functions
   @param x,y - natural coodinates
*/
void dy_bf_quad_4_2d (double *n,double x,double y)
{
  n[0]=0.25*(1.0+x)*(x+y-1.0)+0.25*(1.0+x)*(1.0+y);
  n[1]=0.25*(1.0-x)*(y-x-1.0)+0.25*(1.0-x)*(1.0+y);
  n[2]=0.25*(1.0-x)*(x+y+1.0)-0.25*(1.0-x)*(1.0-y);
  n[3]=0.25*(1.0+x)*(y-x+1.0)-0.25*(1.0+x)*(1.0-y);
  n[4]=0.5*(1.0-x*x);
  n[5]=(x-1.0)*y;
  n[6]=0.5*(x*x-1.0);
  n[7]=(1.0+x)*(-1.0)*y;
}



/**
   function computes modified base functions on triangle for element with rotational DOFs
   
   @param n - array of base functions
   @param l - area coordinates
   @param b,c - coefficients of area coordinates
*/
void bf_rot_3_2d (double *n,double *l,double *b,double *c)
{
  n[0] = l[0];
  n[1] = l[1];
  n[2] = l[2];

  n[3] = (l[0]*l[1]*b[2] - l[2]*l[0]*b[1])/2.0;
  n[4] = (l[1]*l[2]*b[0] - l[0]*l[1]*b[2])/2.0;
  n[5] = (l[2]*l[0]*b[1] - l[1]*l[2]*b[0])/2.0;

  n[6] = (l[0]*l[1]*c[2] - l[2]*l[0]*c[1])/2.0;
  n[7] = (l[1]*l[2]*c[0] - l[0]*l[1]*c[2])/2.0;
  n[8] = (l[2]*l[0]*c[1] - l[1]*l[2]*c[0])/2.0;
}



/**
   function computes derivatives of modified base functions on triangle for element with rotational DOFs with respect of x
   
   @param n - array of base functions
   @param l - area coordinates
   @param b,c - coefficients of area coordinates
*/
void dx_bf_rot_3_2d (double *n,double *l,double *b,double *c)
{
  n[0] = b[0];
  n[1] = b[1];
  n[2] = b[2];

  n[3] = (b[2]*l[1] - b[1]*l[2])*b[0]/2.0;
  n[4] = (b[0]*l[2] - b[2]*l[0])*b[1]/2.0;
  n[5] = (b[1]*l[0] - b[0]*l[1])*b[2]/2.0;

  n[6] = (l[0]*(b[1]*c[2]-b[2]*c[1]) + b[0]*(c[2]*l[1]-c[1]*l[2]))/2.0;
  n[7] = (l[1]*(b[2]*c[0]-b[0]*c[2]) + b[1]*(c[0]*l[2]-c[2]*l[0]))/2.0;
  n[8] = (l[2]*(b[0]*c[1]-b[1]*c[0]) + b[2]*(c[1]*l[0]-c[0]*l[1]))/2.0;
}



/**
   function computes derivatives of modified base functions on triangle for element with rotational DOFs with respect of y
   
   @param n - array of base functions
   @param l - area coordinates
   @param b,c - coefficients of area coordinates
*/
void dy_bf_rot_3_2d (double *n,double *l,double *b,double *c)
{
  n[0] = c[0];
  n[1] = c[1];
  n[2] = c[2];

  n[3] = (l[0]*(c[1]*b[2]-c[2]*b[1]) + c[0]*(b[2]*l[1]-b[1]*l[2]))/2.0;
  n[4] = (l[1]*(c[2]*b[0]-c[0]*b[2]) + c[1]*(b[0]*l[2]-b[2]*l[0]))/2.0;
  n[5] = (l[2]*(c[0]*b[1]-c[1]*b[0]) + c[2]*(b[1]*l[0]-b[0]*l[1]))/2.0;

  n[6] = (c[2]*l[1] - c[1]*l[2])*c[0]/2.0;
  n[7] = (c[0]*l[2] - c[2]*l[0])*c[1]/2.0;
  n[8] = (c[1]*l[0] - c[0]*l[1])*c[2]/2.0;
}



/**
   function computes modified base functions on quadrilateral for element with rotational DOFs
   
   @param n - array of base functions
   @param x,y - natural coordinates
   @param nx,ny - array of x and y components of outer normal vectors
   @param l - array of lengths of element edges
*/
void bf_rot_4_2d (double *n,double x,double y,double *nx,double *ny,double *l)
{
  n[0]  = (1.0+x)*(1.0+y)/4.0;
  n[1]  = (1.0-x)*(1.0+y)/4.0;
  n[2]  = (1.0-x)*(1.0-y)/4.0;
  n[3]  = (1.0+x)*(1.0-y)/4.0;

  n[4]  = ((1.0-y*y)*(1.0+x)*l[3]*nx[3]-(1.0-x*x)*(1.0+y)*l[0]*nx[0])/16.0;
  n[5]  = ((1.0-x*x)*(1.0+y)*l[0]*nx[0]-(1.0-y*y)*(1.0-x)*l[1]*nx[1])/16.0;
  n[6]  = ((1.0-y*y)*(1.0-x)*l[1]*nx[1]-(1.0-x*x)*(1.0-y)*l[2]*nx[2])/16.0;
  n[7]  = ((1.0-x*x)*(1.0-y)*l[2]*nx[2]-(1.0-y*y)*(1.0+x)*l[3]*nx[3])/16.0;

  n[8]  = ((1.0-y*y)*(1.0+x)*l[3]*ny[3]-(1.0-x*x)*(1.0+y)*l[0]*ny[0])/16.0;
  n[9]  = ((1.0-x*x)*(1.0+y)*l[0]*ny[0]-(1.0-y*y)*(1.0-x)*l[1]*ny[1])/16.0;
  n[10] = ((1.0-y*y)*(1.0-x)*l[1]*ny[1]-(1.0-x*x)*(1.0-y)*l[2]*ny[2])/16.0;
  n[11] = ((1.0-x*x)*(1.0-y)*l[2]*ny[2]-(1.0-y*y)*(1.0+x)*l[3]*ny[3])/16.0;
}



/**
   function computes derivatives of modified base functions on quadrilateral for
   element with rotational DOFs with respect of natural coordinate x
   
   @param n - array of base functions
   @param x,y - natural coordinates
   @param nx,ny - array of x and y components of outer normal vectors
   @param l - array of lengths of element edges
*/
void dx_bf_rot_4_2d (double *n,double x,double y,double *nx,double *ny,double *l)
{
  n[0]  = (1.0+y)/4.0;
  n[1]  = (1.0+y)/(-4.0);
  n[2]  = (1.0-y)/(-4.0);
  n[3]  = (1.0-y)/4.0;

  n[4]  = (     (1.0-y*y)*l[3]*nx[3] + 2.0*x*(1.0+y)*l[0]*nx[0])/16.0;
  n[5]  = (-2.0*x*(1.0+y)*l[0]*nx[0] +     (1.0-y*y)*l[1]*nx[1])/16.0;
  n[6]  = (-1.0*(1.0-y*y)*l[1]*nx[1] + 2.0*x*(1.0-y)*l[2]*nx[2])/16.0;
  n[7]  = (-2.0*x*(1.0-y)*l[2]*nx[2] -     (1.0-y*y)*l[3]*nx[3])/16.0;

  n[8]  = (     (1.0-y*y)*l[3]*ny[3] + 2.0*x*(1.0+y)*l[0]*ny[0])/16.0;
  n[9]  = (-2.0*x*(1.0+y)*l[0]*ny[0] +     (1.0-y*y)*l[1]*ny[1])/16.0;
  n[10] = (-1.0*(1.0-y*y)*l[1]*ny[1] + 2.0*x*(1.0-y)*l[2]*ny[2])/16.0;
  n[11] = (-2.0*x*(1.0-y)*l[2]*ny[2] -     (1.0-y*y)*l[3]*ny[3])/16.0;
}



/**
   function computes derivatives of modified base functions on quadrilateral for
   element with rotational DOFs with respect of natural coordinate y
   
   @param n - array of base functions
   @param x,y - natural coordinates
   @param nx,ny - array of x and y components of outer normal vectors
   @param l - array of lengths of element edges
*/
void dy_bf_rot_4_2d (double *n,double x,double y,double *nx,double *ny,double *l)
{
  n[0]  = (1.0+x)/4.0;
  n[1]  = (1.0-x)/4.0;
  n[2]  = (1.0-x)/(-4.0);
  n[3]  = (1.0+x)/(-4.0);

  n[4]  = (-2.0*y*(1.0+x)*l[3]*nx[3] -     (1.0-x*x)*l[0]*nx[0])/16.0;
  n[5]  = (     (1.0-x*x)*l[0]*nx[0] + 2.0*y*(1.0-x)*l[1]*nx[1])/16.0;
  n[6]  = (-2.0*y*(1.0-x)*l[1]*nx[1] +     (1.0-x*x)*l[2]*nx[2])/16.0;
  n[7]  = (-1.0*(1.0-x*x)*l[2]*nx[2] + 2.0*y*(1.0+x)*l[3]*nx[3])/16.0;

  n[8]  = (-2.0*y*(1.0+x)*l[3]*ny[3] -     (1.0-x*x)*l[0]*ny[0])/16.0;
  n[9]  = (     (1.0-x*x)*l[0]*ny[0] + 2.0*y*(1.0-x)*l[1]*ny[1])/16.0;
  n[10] = (-2.0*y*(1.0-x)*l[1]*ny[1] +     (1.0-x*x)*l[2]*ny[2])/16.0;
  n[11] = (-1.0*(1.0-x*x)*l[2]*ny[2] + 2.0*y*(1.0+x)*l[3]*ny[3])/16.0;
}



/**
   function computes base functions of constant curvature triangle
   (finite element for plate problems)
   
   @param n - array of base functions
   @param l - area coordinates
   @param sx,sy - arrays of components
*/
void bf_cct (double *n,double *l,double *sx,double *sy)
{
  n[0]=l[0];
  n[1]=(l[0]*l[1]*sy[0]-l[2]*l[0]*sy[2])/2.0;
  n[2]=(l[2]*l[0]*sx[2]-l[0]*l[1]*sx[0])/2.0;

  n[3]=l[1];
  n[4]=(l[1]*l[2]*sy[1]-l[0]*l[1]*sy[0])/2.0;
  n[5]=(l[0]*l[1]*sx[0]-l[1]*l[2]*sx[1])/2.0;

  n[6]=l[2];
  n[7]=(l[2]*l[0]*sy[2]-l[1]*l[2]*sy[1])/2.0;
  n[8]=(l[1]*l[2]*sx[1]-l[2]*l[0]*sx[2])/2.0;
}



/**
   function computes derivates of base functions of constant curvature triangle with respect of x
   (finite element for plate problems)
   
   @param n - array of base functions
   @param l - area coordinates
   @param b - array of coefficients of area coordinates
   @param sx,sy - arrays of components
*/
void dx_cct (double *n,double *l,double *b,double *sx,double *sy)
{
  n[0]=b[0];
  n[1]=((b[0]*l[1]+l[0]*b[1])*sy[0]-(b[2]*l[0]+l[2]*b[0])*sy[2])/2.0;
  n[2]=((b[2]*l[0]+l[2]*b[0])*sx[2]-(b[0]*l[1]+l[0]*b[1])*sx[0])/2.0;

  n[3]=b[1];
  n[4]=((b[1]*l[2]+l[1]*b[2])*sy[1]-(b[0]*l[1]+l[0]*b[1])*sy[0])/2.0;
  n[5]=((b[0]*l[1]+l[0]*b[1])*sx[0]-(b[1]*l[2]+l[1]*b[2])*sx[1])/2.0;

  n[6]=b[2];
  n[7]=((b[2]*l[0]+l[2]*b[0])*sy[2]-(b[1]*l[2]+l[1]*b[2])*sy[1])/2.0;
  n[8]=((b[1]*l[2]+l[1]*b[2])*sx[1]-(b[2]*l[0]+l[2]*b[0])*sx[2])/2.0;
}



/**
   function computes derivates of base functions of constant curvature triangle with respect of y
   (finite element for plate problems)
   
   @param n - array of base functions
   @param l - area coordinates
   @param b - array of coefficients of area coordinates
   @param sx,sy - arrays of components
*/
void dy_cct (double *n,double *l,double *c,double *sx,double *sy)
{
  n[0]=c[0];
  n[1]=((c[0]*l[1]+l[0]*c[1])*sy[0]-(c[2]*l[0]+l[2]*c[0])*sy[2])/2.0;
  n[2]=((c[2]*l[0]+l[2]*c[0])*sx[2]-(c[0]*l[1]+l[0]*c[1])*sx[0])/2.0;

  n[3]=c[1];
  n[4]=((c[1]*l[2]+l[1]*c[2])*sy[1]-(c[0]*l[1]+l[0]*c[1])*sy[0])/2.0;
  n[5]=((c[0]*l[1]+l[0]*c[1])*sx[0]-(c[1]*l[2]+l[1]*c[2])*sx[1])/2.0;

  n[6]=c[2];
  n[7]=((c[2]*l[0]+l[2]*c[0])*sy[2]-(c[1]*l[2]+l[1]*c[2])*sy[1])/2.0;
  n[8]=((c[1]*l[2]+l[1]*c[2])*sx[1]-(c[2]*l[0]+l[2]*c[0])*sx[2])/2.0;
}



/**
   function computes coefficients a_i of volume coordinates
   L(x,y,z)_i = (a_i + b_i.x + c_i.y + d_i.z)/6/volume
   
   @param a - array of coefficients
   @param x,y,z - array of node coordinates
   @param v - volume of element
*/
void a_coeff_3d (double *a,double *x,double *y,double *z,double v)
{
  a[0] = x[1]*(y[1]*(z[3]-z[2])+y[2]*(z[1]-z[3])+y[3]*(z[2]-z[1]));
  a[0]+= y[1]*(z[1]*(x[3]-x[2])+z[2]*(x[1]-x[3])+z[3]*(x[2]-x[1]));
  a[0]+= z[1]*(x[1]*(y[3]-y[2])+x[2]*(y[1]-y[3])+x[3]*(y[2]-y[1]));
  a[0]/=6.0*v;

  a[1] = x[2]*(y[2]*(z[0]-z[3])+y[3]*(z[2]-z[0])+y[0]*(z[3]-z[2]));
  a[1]+= y[2]*(z[2]*(x[0]-x[3])+z[3]*(x[2]-x[0])+z[0]*(x[3]-x[2]));
  a[1]+= z[2]*(x[2]*(y[0]-y[3])+x[3]*(y[2]-y[0])+x[0]*(y[3]-y[2]));
  a[1]/=6.0*v;

  a[2] = x[3]*(y[3]*(z[1]-z[0])+y[0]*(z[3]-z[1])+y[1]*(z[0]-z[3]));
  a[2]+= y[3]*(z[3]*(x[1]-x[0])+z[0]*(x[3]-x[1])+z[1]*(x[0]-x[3]));
  a[2]+= z[3]*(x[3]*(y[1]-y[0])+x[0]*(y[3]-y[1])+x[1]*(y[0]-y[3]));
  a[2]/=6.0*v;

  a[3] = x[0]*(y[0]*(z[2]-z[1])+y[1]*(z[0]-z[2])+y[2]*(z[1]-z[0]));
  a[3]+= y[0]*(z[0]*(x[2]-x[1])+z[1]*(x[0]-x[2])+z[2]*(x[1]-x[0]));
  a[3]+= z[0]*(x[0]*(y[2]-y[1])+x[1]*(y[0]-y[2])+x[2]*(y[1]-y[0]));
  a[3]/=6.0*v;
}



/**
   function computes coefficients b_i of volume coordinates
   L(x,y,z)_i = (a_i + b_i.x + c_i.y + d_i.z)/6/volume
   
   @param b - array of coefficients
   @param y,z - array of node coordinates
   @param v - volume of element
*/
void b_coeff_3d (double *b,double *y,double *z,double v)
{

  b[0]=-((y[2]-y[1])*(z[3]-z[1])-(y[3]-y[1])*(z[2]-z[1]))/6.0/v;
  b[1]=((y[3]-y[2])*(z[0]-z[2])-(y[0]-y[2])*(z[3]-z[2]))/6.0/v;
  b[2]=-((y[0]-y[3])*(z[1]-z[3])-(y[1]-y[3])*(z[0]-z[3]))/6.0/v;
  b[3]=((y[1]-y[0])*(z[2]-z[0])-(y[2]-y[0])*(z[1]-z[0]))/6.0/v;
/*
  b[0]=((y[1]-y[3])*(z[2]-z[3])-(y[2]-y[3])*(z[1]-z[3]))/6.0/v;
  b[1]=((y[2]-y[0])*(z[3]-z[0])-(y[3]-y[0])*(z[2]-z[0]))/6.0/v;
  b[2]=((y[3]-y[1])*(z[0]-z[1])-(y[0]-y[1])*(z[3]-z[1]))/6.0/v;
  b[3]=((y[0]-y[2])*(z[1]-z[2])-(y[1]-y[2])*(z[0]-z[2]))/6.0/v;
*/
}



/**
   function computes coefficients c_i of volume coordinates
   L(x,y,z)_i = (a_i + b_i.x + c_i.y + d_i.z)/6/volume
   
   @param c - array of coefficients
   @param x,z - array of node coordinates
   @param v - volume of element
*/
void c_coeff_3d (double *c,double *x,double *z,double v)
{

  c[0]=-((x[3]-x[1])*(z[2]-z[1])-(x[2]-x[1])*(z[3]-z[1]))/6.0/v;
  c[1]=((x[0]-x[2])*(z[3]-z[2])-(x[3]-x[2])*(z[0]-z[2]))/6.0/v;
  c[2]=-((x[1]-x[3])*(z[0]-z[3])-(x[0]-x[3])*(z[1]-z[3]))/6.0/v;
  c[3]=((x[2]-x[0])*(z[1]-z[0])-(x[1]-x[0])*(z[2]-z[0]))/6.0/v;
/*
  c[0]=((x[2]-x[3])*(z[1]-z[3])-(x[1]-x[3])*(z[2]-z[3]))/6.0/v;
  c[1]=((x[3]-x[0])*(z[2]-z[0])-(x[2]-x[0])*(z[3]-z[0]))/6.0/v;
  c[2]=((x[0]-x[1])*(z[3]-z[1])-(x[3]-x[1])*(z[0]-z[1]))/6.0/v;
  c[3]=((x[1]-x[2])*(z[0]-z[2])-(x[0]-x[2])*(z[1]-z[2]))/6.0/v;
*/
}



/**
   function computes coefficients d_i of volume coordinates
   L(x,y,z)_i = (a_i + b_i.x + c_i.y + d_i.z)/6/volume
   
   @param d - array of coefficients
   @param x,y - array of node coordinates
   @param v - volume of element
*/
void d_coeff_3d (double *d,double *x,double *y,double v)
{

  d[0]=-((x[2]-x[1])*(y[3]-y[1])-(x[3]-x[1])*(y[2]-y[1]))/6.0/v;
  d[1]=((x[3]-x[2])*(y[0]-y[2])-(x[0]-x[2])*(y[3]-y[2]))/6.0/v;
  d[2]=-((x[0]-x[3])*(y[1]-y[3])-(x[1]-x[3])*(y[0]-y[3]))/6.0/v;
  d[3]=((x[1]-x[0])*(y[2]-y[0])-(x[2]-x[0])*(y[1]-y[0]))/6.0/v;
/*
  d[0]=((x[1]-x[3])*(y[2]-y[3])-(x[2]-x[3])*(y[1]-y[3]))/6.0/v;
  d[1]=((x[2]-x[0])*(y[3]-y[0])-(x[3]-x[0])*(y[2]-y[0]))/6.0/v;
  d[2]=((x[3]-x[1])*(y[0]-y[1])-(x[0]-x[1])*(y[3]-y[1]))/6.0/v;
  d[3]=((x[0]-x[2])*(y[1]-y[2])-(x[1]-x[2])*(y[0]-y[2]))/6.0/v;
*/
}


/**
   function computes coefficients a_i of volume coordinates
   L(x,y,z)_i = a_i + b_i.x + c_i.y + d_i.z
   
   @param a - array of coefficients
   @param x,y,z - array of node coordinates
   @param det - determinat=6*volume
*/
void vola_3d (double *a,double *x,double *y,double *z,double det)
{
  a[0]=(x[1]*(y[2]*z[3]-y[3]*z[2]) + x[2]*(y[3]*z[1]-y[1]*z[3]) + x[3]*(y[1]*z[2]-y[2]*z[1]))/det;
  a[1]=(x[0]*(y[3]*z[2]-y[2]*z[3]) + x[2]*(y[0]*z[3]-y[3]*z[0]) + x[3]*(y[2]*z[0]-y[0]*z[2]))/det;
  a[2]=(x[0]*(y[1]*z[3]-y[3]*z[1]) + x[1]*(y[3]*z[0]-y[0]*z[3]) + x[3]*(y[0]*z[1]-y[1]*z[0]))/det;
  a[3]=(x[0]*(y[2]*z[1]-y[1]*z[2]) + x[1]*(y[0]*z[2]-y[2]*z[0]) + x[2]*(y[1]*z[0]-y[0]*z[1]))/det;
}



/**
   function computes coefficients b_i of volume coordinates
   L(x,y,z)_i = a_i + b_i.x + c_i.y + d_i.z
   
   @param b - array of coefficients
   @param y,z - array of node coordinates
   @param det - determinat=6*volume
*/
void volb_3d (double *b,double *y,double *z,double det)
{
  b[0]=(y[1]*(z[3]-z[2]) + y[2]*(z[1]-z[3]) + y[3]*(z[2]-z[1]))/det;
  b[1]=(y[0]*(z[2]-z[3]) + y[2]*(z[3]-z[0]) + y[3]*(z[0]-z[2]))/det;
  b[2]=(y[0]*(z[3]-z[1]) + y[1]*(z[0]-z[3]) + y[3]*(z[1]-z[0]))/det;
  b[3]=(y[0]*(z[1]-z[2]) + y[1]*(z[2]-z[0]) + y[2]*(z[0]-z[1]))/det;
}



/**
   function computes coefficients c_i of volume coordinates
   L(x,y,z)_i = a_i + b_i.x + c_i.y + d_i.z
   
   @param c - array of coefficients
   @param x,z - array of node coordinates
   @param det - determinat=6*volume
*/
void volc_3d (double *c,double *x,double *z,double det)
{
  c[0]=(x[1]*(z[2]-z[3]) + x[2]*(z[3]-z[1]) + x[3]*(z[1]-z[2]))/det;
  c[1]=(x[0]*(z[3]-z[2]) + x[2]*(z[0]-z[3]) + x[3]*(z[2]-z[0]))/det;
  c[2]=(x[0]*(z[1]-z[3]) + x[1]*(z[3]-z[0]) + x[3]*(z[0]-z[1]))/det;
  c[3]=(x[0]*(z[2]-z[1]) + x[1]*(z[0]-z[2]) + x[2]*(z[1]-z[0]))/det;
}

/**
   function computes coefficients d_i of volume coordinates
   L(x,y,z)_i = a_i + b_i.x + c_i.y + d_i.z
   
   @param d - array of coefficients
   @param x,y - array of node coordinates
   @param det - determinat=6*volume
*/
void vold_3d (double *d,double *x,double *y,double det)
{
  d[0]=(x[1]*(y[3]-y[2]) + x[2]*(y[1]-y[3]) + x[3]*(y[2]-y[1]))/det;
  d[1]=(x[0]*(y[2]-y[3]) + x[2]*(y[3]-y[0]) + x[3]*(y[0]-y[2]))/det;
  d[2]=(x[0]*(y[3]-y[1]) + x[1]*(y[0]-y[3]) + x[3]*(y[1]-y[0]))/det;
  d[3]=(x[0]*(y[1]-y[2]) + x[1]*(y[2]-y[0]) + x[2]*(y[0]-y[1]))/det;
}



/**
   function computes tri-linear base functions on tetrahedron
   
   @param n - array of base functions
   @param x,y,z - natural coodinates
*/
void bf_lin_tet (double *n,double x,double y,double z)
{
  n[0]=x;
  n[1]=y;
  n[2]=z;
  n[3]=1.0-x-y-z;
}



void dx_bf_lin_tet (double *n)
{
  n[0]=1.0;
  n[1]=0.0;
  n[2]=0.0;
  n[3]=-1.0;
}



void dy_bf_lin_tet (double *n)
{
  n[0]=0.0;
  n[1]=1.0;
  n[2]=0.0;
  n[3]=-1.0;
}



void dz_bf_lin_tet (double *n)
{
  n[0]=0.0;
  n[1]=0.0;
  n[2]=1.0;
  n[3]=-1.0;
}



/**
   function computes tri-quadratic base functions on tetrahedron
   
   @param n - array of base functions
   @param x,y,z - natural coodinates
   TKo - 11.2008
*/
void bf_quad_tet(double *n,double x,double y,double z)
{
  n[0]=x*(x-0.5)*2.0;
  n[1]=y*(y-0.5)*2.0;
  n[2]=z*(z-0.5)*2.0;
  n[3]=(1.0-x-y-z)*(0.5-x-y-z)*2.0;
  n[4]=x*y*4.0;
  n[5]=y*z*4.0;
  n[6]=x*z*4.0;
  n[7]=x*(1.0-x-y-z)*4.0;
  n[8]=y*(1.0-x-y-z)*4.0;
  n[9]=z*(1.0-x-y-z)*4.0;
}



/**
   function computes derivatives of tri-quadratic base functions on tetrahedron with respect of natural coordinate x
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
   TKo - 11.2008
*/
void dx_bf_quad_tet(double *n,double x,double y,double z)
{
  n[0] = 4.0*x-1.0;
  n[1] = 0.0;
  n[2] = 0.0;
  n[3] = 4.0*(x+y+z)-3.0;

  n[4] = 4.0*y;
  n[5] = 0.0;
  n[6] = 4.0*z;

  n[7] =  4.0*(1.0-2.0*x-y-z);
  n[8] = -4.0*y;
  n[9] = -4.0*z;
}



/**
   function computes derivatives of tri-quadratic base functions on tetrahedron with respect of natural coordinate y
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
   TKo - 11.2008
*/
void dy_bf_quad_tet(double *n,double x,double y,double z)
{
  n[0] = 0.0;
  n[1] = 4.0*y-1.0;
  n[2] = 0.0;
  n[3] = 4.0*(x+y+z)-3.0;

  n[4] = 4.0*x;
  n[5] = 4.0*z;
  n[6] = 0.0;

  n[7] = -4.0*x;
  n[8] =  4.0*(1.0-x-2.0*y-z);
  n[9] = -4.0*z;
}



/**
   function computes derivatives of tri-quadratic base functions on tetrahedron with respect of natural coordinate z
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
   TKo - 11.2008
*/
void dz_bf_quad_tet (double *n,double x,double y,double z)
{
  n[0] = 0.0;
  n[1] = 0.0;
  n[2] = 4.0*z-1.0;
  n[3] = 4.0*(x+y+z)-3.0;

  n[4] = 0.0;
  n[5] = 4.0*y;
  n[6] = 4.0*x;

  n[7] = -4.0*x;
  n[8] = -4.0*y;
  n[9] =  4.0*(1.0-x-y-2.0*z);
}



/**
   function computes linear base functions on wedge
   
   @param n - array of basis functions
   @param x,y,z - natural coordinates
   
   JK, 9.5.2004
*/
void bf_lin_wed_3d (double *n,double x,double y,double z)
{
  n[0]=x*(1.0+z)/2.0;
  n[1]=y*(1.0+z)/2.0;
  n[2]=(1.0-x-y)*(1.0+z)/2.0;
  n[3]=x*(1.0-z)/2.0;
  n[4]=y*(1.0-z)/2.0;
  n[5]=(1.0-x-y)*(1.0-z)/2.0;
}

/**
   function computes derivatives of linear base functions on wedge with respect to x
   
   @param n - array of basis functions
   @param z - natural coordinate
   
   JK, 9.5.2004
*/
void dx_bf_lin_wed_3d (double *n,double z)
{
  n[0]=(1.0+z)/2.0;
  n[1]=0.0;
  n[2]=-1.0*(1.0+z)/2.0;
  n[3]=(1.0-z)/2.0;
  n[4]=0.0;
  n[5]=-1.0*(1.0-z)/2.0;
}

/**
   function computes derivatives of linear base functions on wedge with respect to y
   
   @param n - array of basis functions
   @param z - natural coordinate
   
   JK, 9.5.2004
*/
void dy_bf_lin_wed_3d (double *n,double z)
{
  n[0]=0.0;
  n[1]=(1.0+z)/2.0;
  n[2]=-1.0*(1.0+z)/2.0;
  n[3]=0.0;
  n[4]=(1.0-z)/2.0;
  n[5]=-1.0*(1.0-z)/2.0;
}

/**
   function computes derivatives of linear base functions on wedge with respect to z
   
   @param n - array of basis functions
   @param x,y - natural coordinates
   
   JK, 9.5.2004
*/
void dz_bf_lin_wed_3d (double *n,double x,double y)
{
  n[0]=x/2.0;
  n[1]=y/2.0;
  n[2]=(1.0-x-y)/2.0;
  n[3]=x/(-2.0);
  n[4]=y/(-2.0);
  n[5]=(1.0-x-y)/(-2.0);
}






/**
   function computes quadratic approximation functions on wedge
   
   @param n - array of basis functions
   @param x,y,z - natural coordinates
   
   JK, 19.9.2004
*/
void bf_quad_wed_3d (double *n,double x,double y,double z)
{
  n[0]  = x*(x-0.5)*(1.0+z)*z;
  n[1]  = y*(y-0.5)*(1.0+z)*z;
  n[2]  = (1.0-x-y)*(0.5-x-y)*(1.0+z)*z;
  n[3]  = x*(x-0.5)*(z-1.0)*z;
  n[4]  = y*(y-0.5)*(z-1.0)*z;
  n[5]  = (1.0-x-y)*(0.5-x-y)*(z-1.0)*z;
  n[6]  = 2.0*x*y*(1.0+z);
  n[7]  = 2.0*y*(1.0-x-y)*(1.0+z);
  n[8]  = 2.0*x*(1.0-x-y)*(1.0+z);
  n[9]  = x*(1.0-z*z);
  n[10] = y*(1.0-z*z);
  n[11] = (1.0-x-y)*(1.0-z*z);
  n[12] = 2.0*x*y*(1.0-z);
  n[13] = 2.0*y*(1.0-x-y)*(1.0-z);
  n[14] = 2.0*x*(1.0-x-y)*(1.0-z);
}

/**
   function computes derivatives of linear approximation functions on wedge with respect to x
   
   @param n - array of basis functions
   @param x,y,z - natural coordinates
   
   JK, 19.9.2004
*/
void dx_bf_quad_wed_3d (double *n,double x,double y,double z)
{
  n[0]  = (2.0*x-0.5)*(1.0+z)*z;
  n[1]  = 0.0;
  n[2]  = (2.0*x+2.0*y-1.5)*(1.0+z)*z;
  n[3]  = (2.0*x-0.5)*(z-1.0)*z;
  n[4]  = 0.0;
  n[5]  = (2.0*x+2.0*y-1.5)*(z-1.0)*z;
  n[6]  = 2.0*y*(1.0+z);
  n[7]  = -2.0*y*(1.0+z);
  n[8]  = (2.0-4.0*x-2.0*y)*(1.0+z);
  n[9]  = (1.0-z*z);
  n[10] = 0.0;
  n[11] = z*z-1.0;
  n[12] = 2.0*y*(1.0-z);
  n[13] = 2.0*y*(z-1.0);
  n[14] = (2.0-4.0*x-2.0*y)*(1.0-z);
}

/**
   function computes derivatives of quadratic approximation functions on wedge with respect to y
   
   @param n - array of basis functions
   @param x,y,z - natural coordinates
   
   JK, 19.9.2004
*/
void dy_bf_quad_wed_3d (double *n,double x,double y,double z)
{
  n[0]  = 0.0;
  n[1]  = (2.0*y-0.5)*(1.0+z)*z;
  n[2]  = (2.0*x+2.0*y-1.5)*(1.0+z)*z;
  n[3]  = 0.0;
  n[4]  = (2.0*y-0.5)*(z-1.0)*z;
  n[5]  = (2.0*x+2.0*y-1.5)*(z-1.0)*z;
  n[6]  = 2.0*x*(1.0+z);
  n[7]  = (2.0-2.0*x-4.0*y)*(1.0+z);
  n[8]  = -2.0*x*(1.0+z);
  n[9]  = 0.0;
  n[10] = 1.0-z*z;
  n[11] = z*z-1.0;
  n[12] = 2.0*x*(1.0-z);
  n[13] = (2.0-2.0*x-4.0*y)*(1.0-z);
  n[14] = 2.0*x*(z-1.0);
}

/**
   function computes derivatives of quadratic approximation functions on wedge with respect to z
   
   @param n - array of basis functions
   @param x,y,z - natural coordinates
   
   JK, 19.9.2004
*/
void dz_bf_quad_wed_3d (double *n,double x,double y,double z)
{
  n[0]  = x*(x-0.5)*(1.0+2.0*z);
  n[1]  = y*(y-0.5)*(1.0+2.0*z);
  n[2]  = (1.0-x-y)*(0.5-x-y)*(1.0+2.0*z);
  n[3]  = x*(x-0.5)*(2.0*z-1.0);
  n[4]  = y*(y-0.5)*(2.0*z-1.0);
  n[5]  = (1.0-x-y)*(0.5-x-y)*(2.0*z-1.0);
  n[6]  = 2.0*x*y;
  n[7]  = 2.0*y*(1.0-x-y);
  n[8]  = 2.0*x*(1.0-x-y);
  n[9]  = -2.0*x*z;
  n[10] = -2.0*y*z;
  n[11] = -2.0*z*(1.0-x-y);
  n[12] = -2.0*x*y;
  n[13] = -2.0*y*(1.0-x-y);
  n[14] = -2.0*x*(1.0-x-y);
}







/**
   function computes tri-linear base functions on hexahedron
   
   @param n - array of base functions
   @param x,y,z - natural coodinates
*/
void bf_lin_hex_3d (double *n,double x,double y,double z)
{
  n[0]=(1.0+x)*(1.0+y)*(1.0+z)/8.0;
  n[1]=(1.0-x)*(1.0+y)*(1.0+z)/8.0;
  n[2]=(1.0-x)*(1.0-y)*(1.0+z)/8.0;
  n[3]=(1.0+x)*(1.0-y)*(1.0+z)/8.0;
  n[4]=(1.0+x)*(1.0+y)*(1.0-z)/8.0;
  n[5]=(1.0-x)*(1.0+y)*(1.0-z)/8.0;
  n[6]=(1.0-x)*(1.0-y)*(1.0-z)/8.0;
  n[7]=(1.0+x)*(1.0-y)*(1.0-z)/8.0;
}

/**
   function computes derivatives of tri-linear base functions on hexahedron with respect of natural coordinate x
   
   @param n - array of derivatives of base functions
   @param y,z - natural coodinates
*/
void dx_bf_lin_hex_3d (double *n,double y,double z)
{
  n[0]=(1.0+y)*(1.0+z)/8.0;
  n[1]=(1.0+y)*(1.0+z)/(-8.0);
  n[2]=(1.0-y)*(1.0+z)/(-8.0);
  n[3]=(1.0-y)*(1.0+z)/8.0;
  n[4]=(1.0+y)*(1.0-z)/8.0;
  n[5]=(1.0+y)*(1.0-z)/(-8.0);
  n[6]=(1.0-y)*(1.0-z)/(-8.0);
  n[7]=(1.0-y)*(1.0-z)/8.0;
}



/**
   function computes derivatives of tri-linear base functions on hexahedron with respect of natural coordinate y
   
   @param n - array of derivatives of base functions
   @param x,z - natural coodinates
*/
void dy_bf_lin_hex_3d (double *n,double x,double z)
{
  n[0]=(1.0+x)*(1.0+z)/8.0;     
  n[1]=(1.0-x)*(1.0+z)/8.0;
  n[2]=(1.0-x)*(1.0+z)/(-8.0);
  n[3]=(1.0+x)*(1.0+z)/(-8.0);
  n[4]=(1.0+x)*(1.0-z)/8.0;
  n[5]=(1.0-x)*(1.0-z)/8.0;
  n[6]=(1.0-x)*(1.0-z)/(-8.0);
  n[7]=(1.0+x)*(1.0-z)/(-8.0);
}



/**
   function computes derivatives of tri-linear base functions on hexahedron with respect of natural coordinate z
   
   @param n - array of derivatives of base functions
   @param x,y - natural coodinates
*/
void dz_bf_lin_hex_3d (double *n,double x,double y)
{
  n[0]=(1.0+x)*(1.0+y)/8.0;
  n[1]=(1.0-x)*(1.0+y)/8.0;
  n[2]=(1.0-x)*(1.0-y)/8.0;
  n[3]=(1.0+x)*(1.0-y)/8.0;
  n[4]=(1.0+x)*(1.0+y)/(-8.0);
  n[5]=(1.0-x)*(1.0+y)/(-8.0);
  n[6]=(1.0-x)*(1.0-y)/(-8.0);
  n[7]=(1.0+x)*(1.0-y)/(-8.0);
}



/**
   function computes tri-quadratic base functions on hexahedron
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
*/
void bf_quad_hex_3d (double *n,double x,double y,double z)
{
  n[0]  = (1.0+x)*(1.0+y)*(1.0+z)*(x+y+z-2.0)/8.0;
  n[1]  = (1.0-x)*(1.0+y)*(1.0+z)*(y-x+z-2.0)/8.0;
  n[2]  = (1.0-x)*(1.0-y)*(1.0+z)*(z-x-y-2.0)/8.0;
  n[3]  = (1.0+x)*(1.0-y)*(1.0+z)*(x-y+z-2.0)/8.0;

  n[4]  = (1.0+x)*(1.0+y)*(1.0-z)*(x+y-z-2.0)/8.0;
  n[5]  = (1.0-x)*(1.0+y)*(1.0-z)*(y-x-z-2.0)/8.0;
  n[6]  = (1.0-x)*(1.0-y)*(1.0-z)*(x+y+z+2.0)/(-8.0);
  n[7]  = (1.0+x)*(1.0-y)*(1.0-z)*(x-y-z-2.0)/8.0;

  n[8]  = (1.0-x*x)*(1.0+y)*(1.0+z)/4.0;
  n[9]  = (1.0-y*y)*(1.0-x)*(1.0+z)/4.0;
  n[10] = (1.0-x*x)*(1.0-y)*(1.0+z)/4.0;
  n[11] = (1.0-y*y)*(1.0+x)*(1.0+z)/4.0;

  n[12] = (1.0+x)*(1.0+y)*(1.0-z*z)/4.0;
  n[13] = (1.0-x)*(1.0+y)*(1.0-z*z)/4.0;
  n[14] = (1.0-x)*(1.0-y)*(1.0-z*z)/4.0;
  n[15] = (1.0+x)*(1.0-y)*(1.0-z*z)/4.0;

  n[16] = (1.0-x*x)*(1.0+y)*(1.0-z)/4.0;
  n[17] = (1.0-y*y)*(1.0-x)*(1.0-z)/4.0;
  n[18] = (1.0-x*x)*(1.0-y)*(1.0-z)/4.0;
  n[19] = (1.0-y*y)*(1.0+x)*(1.0-z)/4.0;
}



/**
   function computes derivatives of tri-quadratic base functions on hexahedron with respect of natural coordinate x
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
*/
void dx_bf_quad_hex_3d (double *n,double x,double y,double z)
{
  n[0]  = ((1.0+y)*(1.0+z)*(x+y+z-2.0) + (1.0+x)*(1.0+y)*(1.0+z))/8.0;
  n[1]  = ((1.0+y)*(1.0+z)*(y-x+z-2.0) + (1.0-x)*(1.0+y)*(1.0+z))/(-8.0);
  n[2]  = ((1.0-y)*(1.0+z)*(z-x-y-2.0) + (1.0-x)*(1.0-y)*(1.0+z))/(-8.0);
  n[3]  = ((1.0-y)*(1.0+z)*(x-y+z-2.0) + (1.0+x)*(1.0-y)*(1.0+z))/8.0;

  n[4]  = ((1.0+y)*(1.0-z)*(x+y-z-2.0) + (1.0+x)*(1.0+y)*(1.0-z))/8.0;
  n[5]  = ((1.0+y)*(1.0-z)*(y-x-z-2.0) + (1.0-x)*(1.0+y)*(1.0-z))/(-8.0);
  n[6]  = ((1.0-y)*(1.0-z)*(x+y+z+2.0) - (1.0-x)*(1.0-y)*(1.0-z))/8.0;
  n[7]  = ((1.0-y)*(1.0-z)*(x-y-z-2.0) + (1.0+x)*(1.0-y)*(1.0-z))/8.0;

  n[8]  = x*(1.0+y)*(1.0+z)/(-2.0);
  n[9]  = (1.0-y*y)*(1.0+z)/(-4.0);
  n[10] = x*(1.0-y)*(1.0+z)/(-2.0);
  n[11] = (1.0-y*y)*(1.0+z)/4.0;

  n[12] = (1.0+y)*(1.0-z*z)/4.0;
  n[13] = (1.0+y)*(1.0-z*z)/(-4.0);
  n[14] = (1.0-y)*(1.0-z*z)/(-4.0);
  n[15] = (1.0-y)*(1.0-z*z)/4.0;

  n[16] = x*(1.0+y)*(1.0-z)/(-2.0);
  n[17] = (1.0-y*y)*(1.0-z)/(-4.0);
  n[18] = x*(1.0-y)*(1.0-z)/(-2.0);
  n[19] = (1.0-y*y)*(1.0-z)/4.0;
}



/**
   function computes derivatives of tri-quadratic base functions on hexahedron with respect of natural coordinate y
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
*/
void dy_bf_quad_hex_3d (double *n,double x,double y,double z)
{
  n[0]  = ((1.0+x)*(1.0+z)*(x+y+z-2.0) + (1.0+x)*(1.0+y)*(1.0+z))/8.0;
  n[1]  = ((1.0-x)*(1.0+z)*(y-x+z-2.0) + (1.0-x)*(1.0+y)*(1.0+z))/8.0;
  n[2]  = ((1.0-x)*(1.0+z)*(z-x-y-2.0) + (1.0-x)*(1.0-y)*(1.0+z))/(-8.0);
  n[3]  = ((1.0+x)*(1.0+z)*(x-y+z-2.0) + (1.0+x)*(1.0-y)*(1.0+z))/(-8.0);

  n[4]  = ((1.0+x)*(1.0-z)*(x+y-z-2.0) + (1.0+x)*(1.0+y)*(1.0-z))/8.0;
  n[5]  = ((1.0-x)*(1.0-z)*(y-x-z-2.0) + (1.0-x)*(1.0+y)*(1.0-z))/8.0;
  n[6]  = ((1.0-x)*(1.0-z)*(x+y+z+2.0) - (1.0-x)*(1.0-y)*(1.0-z))/8.0;
  n[7]  = ((1.0+x)*(1.0-z)*(x-y-z-2.0) + (1.0+x)*(1.0-y)*(1.0-z))/(-8.0);

  n[8]  = (1.0-x*x)*(1.0+z)/4.0;
  n[9]  = y*(1.0-x)*(1.0+z)/(-2.0);
  n[10] = (1.0-x*x)*(1.0+z)/(-4.0);
  n[11] = y*(1.0+x)*(1.0+z)/(-2.0);

  n[12] = (1.0+x)*(1.0-z*z)/4.0;
  n[13] = (1.0-x)*(1.0-z*z)/4.0;
  n[14] = (1.0-x)*(1.0-z*z)/(-4.0);
  n[15] = (1.0+x)*(1.0-z*z)/(-4.0);

  n[16] = (1.0-x*x)*(1.0-z)/4.0;
  n[17] = y*(1.0-x)*(1.0-z)/(-2.0);
  n[18] = (1.0-x*x)*(1.0-z)/(-4.0);
  n[19] = y*(1.0+x)*(1.0-z)/(-2.0);
}

/**
   function computes derivatives of tri-quadratic base functions on hexahedron with respect of natural coordinate z
   
   @param n - array of derivatives of base functions
   @param x,y,z - natural coodinates
*/
void dz_bf_quad_hex_3d (double *n,double x,double y,double z)
{
  n[0]  = ((1.0+x)*(1.0+y)*(x+y+z-2.0) + (1.0+x)*(1.0+y)*(1.0+z))/8.0;
  n[1]  = ((1.0-x)*(1.0+y)*(y-x+z-2.0) + (1.0-x)*(1.0+y)*(1.0+z))/8.0;
  n[2]  = ((1.0-x)*(1.0-y)*(z-x-y-2.0) + (1.0-x)*(1.0-y)*(1.0+z))/8.0;
  n[3]  = ((1.0+x)*(1.0-y)*(x-y+z-2.0) + (1.0+x)*(1.0-y)*(1.0+z))/8.0;

  n[4]  = ((1.0+x)*(1.0+y)*(x+y-z-2.0) + (1.0+x)*(1.0+y)*(1.0-z))/(-8.0);
  n[5]  = ((1.0-x)*(1.0+y)*(y-x-z-2.0) + (1.0-x)*(1.0+y)*(1.0-z))/(-8.0);
  n[6]  = ((1.0-x)*(1.0-y)*(x+y+z+2.0) - (1.0-x)*(1.0-y)*(1.0-z))/8.0;
  n[7]  = ((1.0+x)*(1.0-y)*(x-y-z-2.0) + (1.0+x)*(1.0-y)*(1.0-z))/(-8.0);

  n[8]  = (1.0-x*x)*(1.0+y)/4.0;
  n[9]  = (1.0-y*y)*(1.0-x)/4.0;
  n[10] = (1.0-x*x)*(1.0-y)/4.0;
  n[11] = (1.0-y*y)*(1.0+x)/4.0;

  n[12] = (1.0+x)*(1.0+y)*z/(-2.0);
  n[13] = (1.0-x)*(1.0+y)*z/(-2.0);
  n[14] = (1.0-x)*(1.0-y)*z/(-2.0);
  n[15] = (1.0+x)*(1.0-y)*z/(-2.0);

  n[16] = (1.0-x*x)*(1.0+y)/(-4.0);
  n[17] = (1.0-y*y)*(1.0-x)/(-4.0);
  n[18] = (1.0-x*x)*(1.0-y)/(-4.0);
  n[19] = (1.0-y*y)*(1.0+x)/(-4.0);
}


void bf_quad_hexrot_3d (double *n,double x,double y,double z)
{
  
  n[0]  = (1.0-x*x)*(1.0+y)*(1.0+z)/4.0;
  n[1]  = (1.0-y*y)*(1.0-x)*(1.0+z)/4.0;
  n[2] = (1.0-x*x)*(1.0-y)*(1.0+z)/4.0;
  n[3] = (1.0-y*y)*(1.0+x)*(1.0+z)/4.0;

  n[4] = (1.0+x)*(1.0+y)*(1.0-z*z)/4.0;
  n[5] = (1.0-x)*(1.0+y)*(1.0-z*z)/4.0;
  n[6] = (1.0-x)*(1.0-y)*(1.0-z*z)/4.0;
  n[7] = (1.0+x)*(1.0-y)*(1.0-z*z)/4.0;

  n[8] = (1.0-x*x)*(1.0+y)*(1.0-z)/4.0;
  n[9] = (1.0-y*y)*(1.0-x)*(1.0-z)/4.0;
  n[10] = (1.0-x*x)*(1.0-y)*(1.0-z)/4.0;
  n[11] = (1.0-y*y)*(1.0+x)*(1.0-z)/4.0;
}

void dx_bf_quad_hexrot_3d (double *n,double x,double y,double z)
{
  //  side 1-2, 2-3, 3-4, 4-1,  1-5, 2-6, 3-7, 4-8,  5-6, 6-7, 7-8, 8-5,   

  n[0]  = x*(1.0+y)*(1.0+z)/(-2.0);
  n[1]  = (1.0-y*y)*(1.0+z)/(-4.0);
  n[2] = x*(1.0-y)*(1.0+z)/(-2.0);
  n[3] = (1.0-y*y)*(1.0+z)/4.0;

  n[4] = (1.0+y)*(1.0-z*z)/4.0;
  n[5] = (1.0+y)*(1.0-z*z)/(-4.0);
  n[6] = (1.0-y)*(1.0-z*z)/(-4.0);
  n[7] = (1.0-y)*(1.0-z*z)/4.0;

  n[8] = x*(1.0+y)*(1.0-z)/(-2.0);
  n[9] = (1.0-y*y)*(1.0-z)/(-4.0);
  n[10] = x*(1.0-y)*(1.0-z)/(-2.0);
  n[11] = (1.0-y*y)*(1.0-z)/4.0;
}
void dy_bf_quad_hexrot_3d (double *n,double x,double y,double z)
{
  //  side 1-2, 2-3, 3-4, 4-1,  1-5, 2-6, 3-7, 4-8,  5-6, 6-7, 7-8, 8-5,   

  n[0]  = (1.0-x*x)*(1.0+z)/4.0;
  n[1]  = y*(1.0-x)*(1.0+z)/(-2.0);
  n[2] = (1.0-x*x)*(1.0+z)/(-4.0);
  n[3] = y*(1.0+x)*(1.0+z)/(-2.0);

  n[4] = (1.0+x)*(1.0-z*z)/4.0;
  n[5] = (1.0-x)*(1.0-z*z)/4.0;
  n[6] = (1.0-x)*(1.0-z*z)/(-4.0);
  n[7] = (1.0+x)*(1.0-z*z)/(-4.0);

  n[8] = (1.0-x*x)*(1.0-z)/4.0;
  n[9] = y*(1.0-x)*(1.0-z)/(-2.0);
  n[10] = (1.0-x*x)*(1.0-z)/(-4.0);
  n[11] = y*(1.0+x)*(1.0-z)/(-2.0);

}
void dz_bf_quad_hexrot_3d (double *n,double x,double y,double z)
{
  //  side 1-2, 2-3, 3-4, 4-1,  1-5, 2-6, 3-7, 4-8,  5-6, 6-7, 7-8, 8-5,   

  n[0]  = (1.0-x*x)*(1.0+y)/4.0;
  n[1]  = (1.0-y*y)*(1.0-x)/4.0;
  n[2] = (1.0-x*x)*(1.0-y)/4.0;
  n[3] = (1.0-y*y)*(1.0+x)/4.0;

  n[4] = (1.0+x)*(1.0+y)*z/(-2.0);
  n[5] = (1.0-x)*(1.0+y)*z/(-2.0);
  n[6] = (1.0-x)*(1.0-y)*z/(-2.0);
  n[7] = (1.0+x)*(1.0-y)*z/(-2.0);

  n[8] = (1.0-x*x)*(1.0+y)/(-4.0);
  n[9] = (1.0-y*y)*(1.0-x)/(-4.0);
  n[10] = (1.0-x*x)*(1.0-y)/(-4.0);
  n[11] = (1.0-y*y)*(1.0+x)/(-4.0);
}
