#include "tensor.h"
#include <math.h>
#include <stdlib.h>

/**
   function computes deviatoric part of the tensor
   
   plane stress cases
   vector tens contains 3 components
   tens[0]=t_11, tens[1]=t_22, tens[2]=t_12,
   
   3D cases
   vector tens contains 6 components
   tens[0]=t_11, tens[1]=t_22, tens[2]=t_33,
   tens[3]=t_23, tens[4]=t_31, tens[5]=t_12,
   
   28.10.2001
*/
void deviator (matrix &tens,matrix &dev)
{
  double isopart;
  
  //  isotropic part of the tensor
  isopart = first_invar (tens)/3.0;
  
  //  deviatoric part of the tensor
  copym (tens,dev);
  dev[0][0]=tens[0][0]-isopart;
  dev[1][1]=tens[1][1]-isopart;
  dev[2][2]=tens[2][2]-isopart;
}

/**
   function computes normed deviatoric part of the tensor
   function overwrites tens by its normed deviatoric part

   plane stress cases
   vector tens contains 3 components
   tens[0]=t_11, tens[1]=t_22, tens[2]=t_12,

   3D cases
   vector tens contains 6 components
   tens[0]=t_11, tens[1]=t_22, tens[2]=t_33,
   tens[3]=t_23, tens[4]=t_31, tens[5]=t_12,
   
   28.10.2001
*/
void normedtensor (matrix &tens,matrix &ntens)
{
  double norm,zero;
  zero=1.0e-10;
  norm = tensornorm (tens);
  copym (tens,ntens);
  if (norm<zero){}
  else{
    cmulm (1.0/norm,ntens);
  }
}

/**
   function computes norm of second order tensor

   29.10.2001
*/
double tensornorm (matrix &tens)
{
  long i,j;
  double nor;
  
  nor=0.0;
  for (i=0;i<3;i++){
    for (j=0;j<3;j++){
      nor+=tens[i][j]*tens[i][j];
    }
  }
  nor=sqrt(nor);
  return nor;
}

/**
   function computes cumulative strain
   
   @param eps - vector containing strain increments
   @param cs - cumulative strain
   29.10.2001
*/
void cumulstrain (matrix &eps,double &cs)
{
  cs+=sqrt(2.0/3.0)*tensornorm (eps);
}

/**
   function computes first invariant
   4.1.2002
*/
double first_invar (matrix &tens)
{
  double inv;
  inv=tens[0][0]+tens[1][1]+tens[2][2];
  return inv;
}

/**
   The function computes negative value of second invariant.
   Negative value is used due to usual usage of this function
   in second invariant of stress deviator which is defined with a
   negative sign (energy of shape change)

   @param tens - %matrix (3,3) with tensor values
   4.1.2002
*/
double second_invar (matrix &tens)
{
  double inv;
  inv=tens[0][0]*tens[1][1]-tens[0][1]*tens[1][0];
  inv+=tens[0][0]*tens[2][2]-tens[0][2]*tens[2][0];
  inv+=tens[1][1]*tens[2][2]-tens[1][2]*tens[2][1];
  return (inv*(-1.0));
}

/**
   function computes third invariant
   4.1.2002
*/
double third_invar (matrix &tens)
{
  double inv;
  inv=tens[0][0]*tens[1][1]*tens[2][2]+tens[0][1]*tens[1][2]*tens[2][0]+tens[0][2]*tens[1][0]*tens[2][1];
  inv-=tens[0][2]*tens[1][1]*tens[2][0]+tens[1][2]*tens[2][1]*tens[0][0]+tens[2][2]*tens[0][1]*tens[1][0];
  return inv;
}


/*
  function transforms second order tensor tens:

  tmat ...transformation matrix
                 
  then tmt is transposed tmat

  and tensloc=tmt*tensglob*tmat

  plane stress
  vector tens contains 3 components
  eps[0]=eps_11, eps[1]=eps_22, eps[2]=eps_12,

  plane strains
  vector tens contains 4 components
  eps[0]=eps_11, eps[1]=eps_22, eps[2]=eps_33, eps[3]=eps_12,

  3D cases
  vector sig contains 6 components
  eps[0]=eps_11, eps[1]=eps_22, eps[2]=eps_33,
  eps[3]=eps_23, eps[4]=eps_31, eps[5]=eps_12,

 17.1.2002
*/

void glob_loc_tens_trans (vector &tens, matrix &tmat)
{
  int n;
  
  switch ( tens.n )
    {
    case 3:
      if ( tmat.n != 2 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      if ( tmat.m != 2 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      n=2;
      break;
      
    case 4:
      if ( tmat.n != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      if ( tmat.m != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      
      n=3;
      break;
      
    case 6:
      if ( tmat.n != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      if ( tmat.m != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      
      n=3;
      break;
      
    default :
      fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
    }
  
  matrix a(n,n);
  matrix b(n,n);
  
  
  switch ( tens.n )
    {
    case 3 :
      
      a[0][0]=tens[0];
      a[1][1]=tens[1];
      a[0][1]=tens[2];
      a[1][0]=tens[2];
      mtxm(tmat,a,b);
      mxm(b,tmat,a);
      tens[0]=a[0][0];
      tens[1]=a[1][1];
      tens[2]=a[0][1];
      break;
      
    case 4 :
      a[0][0]=tens[0];
      a[1][1]=tens[1];
      a[2][2]=tens[2];
      a[1][2]=tens[3];
      a[2][1]=tens[3];
      mtxm(tmat,a,b);
      mxm(b,tmat,a);
      tens[0]=a[0][0];
      tens[1]=a[1][1];
      tens[2]=a[2][2];
      tens[3]=a[1][2];
      break;
      
    case 6 :
      a[0][0]=tens[0];
      a[1][1]=tens[1];
      a[2][2]=tens[2];
      a[1][2]=tens[3];
      a[2][1]=tens[3];
      a[0][2]=tens[4];
      a[2][0]=tens[4];
      a[0][1]=tens[5];
      a[1][0]=tens[5];
      mtxm(tmat,a,b);
      mxm(b,tmat,a);
      tens[0]=a[0][0];
      tens[1]=a[1][1];
      tens[2]=a[2][2];
      tens[3]=a[1][2];
      tens[4]=a[0][2];
      tens[5]=a[0][1];
      break;
    }
}


/*
  function transforms second order tensor tens:

  tmat ...transformation %matrix
                 
  then tmt is transposed tmat

  and tensglob=tmat*tensloc*tmt

  plane stress
  vector tens contains 3 components
  eps[0]=eps_11, eps[1]=eps_22, eps[2]=eps_12,

  plane strains
  vector tens contains 4 components
  eps[0]=eps_11, eps[1]=eps_22, eps[2]=eps_33, eps[3]=eps_12,

  3D cases
  vector sig contains 6 components
  eps[0]=eps_11, eps[1]=eps_22, eps[2]=eps_33,
  eps[3]=eps_23, eps[4]=eps_31, eps[5]=eps_12,

 17.1.2002
*/

void loc_glob_tens_trans (vector &tens, matrix &tmat)
{
  int n;
  
  switch ( tens.n )
    {
    case 3:
      if ( tmat.n != 2 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      if ( tmat.m != 2 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      n=2;
      break;
      
    case 4:
      if ( tmat.n != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      if ( tmat.m != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      
      n=3;
      break;
      
    case 6:
      if ( tmat.n != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      if ( tmat.m != 3 ) fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
      
      n=3;
      break;
      
    default :
      fprintf(stderr,"\nError multiplying matrix by vector - incompatible size (tens_trans in file tensor.*)\n");
    }
  
  matrix a(n,n);
  matrix b(n,n);
  
  
  switch ( tens.n )
    {
    case 3 :
      
      a[0][0]=tens[0];
      a[1][1]=tens[1];
      a[0][1]=tens[2];
      a[1][0]=tens[2];
      mxm(tmat,a,b);
      mxmt(b,tmat,a);
      tens[0]=a[0][0];
      tens[1]=a[1][1];
      tens[2]=a[0][1];
      break;
      
    case 4 :
      a[0][0]=tens[0];
      a[1][1]=tens[1];
      a[2][2]=tens[2];
      a[1][2]=tens[3];
      a[2][1]=tens[3];
      mxm(tmat,a,b);
      mxmt(b,tmat,a);
      tens[0]=a[0][0];
      tens[1]=a[1][1];
      tens[2]=a[2][2];
      tens[3]=a[1][2];
      break;
      
    case 6 :
      a[0][0]=tens[0];
      a[1][1]=tens[1];
      a[2][2]=tens[2];
      a[1][2]=tens[3];
      a[2][1]=tens[3];
      a[0][2]=tens[4];
      a[2][0]=tens[4];
      a[0][1]=tens[5];
      a[1][0]=tens[5];
      mxm(tmat,a,b);
      mxmt(b,tmat,a);
      tens[0]=a[0][0];
      tens[1]=a[1][1];
      tens[2]=a[2][2];
      tens[3]=a[1][2];
      tens[4]=a[0][2];
      tens[5]=a[0][1];
      break;
    }
}

/**
   function computes inner product of two second order tensors
   it computes a_ij b_ij
   
   @param a,b - second order tensors
   
   JK, 9.8.2005
*/
double tensor2_inner_prod (matrix &a,matrix &b)
{
  long i,j,n;
  double ip;
  
  n=a.n;
  if (a.m!=3 || a.n!=3 || b.m!=3 || b.n!=3){
    fprintf (stderr,"\n\n incompatible tensor(s) in function tensor2_inner_prod (file %s, line %d),\n",__FILE__,__LINE__);
  }
  
  ip=0.0;
  for (i=0;i<n;i++){
    for (j=0;j<n;j++){
      ip+=a[i][j]*b[i][j];
    }
  }
  
  return ip;
}


/** 
    Function computes f(Aij), where f(x) is function of real and 
    Aij is second order tensor.
    @param a - matrix containing second order tensor
    @param f - pointer to function with one double argument and returning double
    @param nijac - number of iterations in Jacobi method
    @param error - required error of principal values
    @param zero - computer zero
    @param af - matrix containing resulting second order tensor
  

*/
void f_tensor(matrix &a, double (*f)(double), long nijac, double error, double zero, matrix &af)
{
  vector pa(3);
  matrix t(3,3);
  long i;
  princ_val (a, pa, t, nijac, error, zero, 3, 1);
  fillm(0.0, af);
  for(i=0; i<3; i++)
    af[i][i] = (*f)(pa[i]);
  lgmatrixtransf(af, t);  
}



/** 
    Function computes f(Aij), where f(x) is function of real and 
    Aij is second order tensor. The principal directions are known 
    @param a - matrix containing second order tensor
    @param f - pointer to function with one double argument and returning double
    @param t - transformation matrix to the principal directions
    @param af - matrix containing resulting second order tensor

*/
void f_tensor(matrix &a, double (*f)(double), matrix &t, matrix &af)
{
  long i;
  copym(a, af);
  glmatrixtransf(af, t);  
  fillm(0.0, af);
  for(i=0; i<3; i++)
    af[i][i] = (*f)(a[i][i]);
  lgmatrixtransf(af, t);  
}



/** 
    Function computes derivatives of i-th component of alpha-th principal direction vector n of tensor A
    with respect to components of tensor A ie. d(n_{i,alpha})/d(A_{kl}) = (dpd_da)_{kl}
    Aij is second order tensor. The principal directions are known 
    @param t - matrix containing principal direction vectors stored in columns ie. x_g = T x_l
    @param pa - vector of principal values of A
    @param alpha - index of principal direction (index of column in t)
    @param i - index of vector component
    @param dpd_da - matrix containing resulting second order tensor

*/
long dpdir_da(matrix &t, vector &pa, long alpha, long i, matrix &dpd_da)
{
  long k,l,beta,ret;

  ret = 0;
  fillm(0.0, dpd_da);
  for(k=0; k<3; k++)
  {
    for(l=0; l<3; l++)
    {
      for(beta=0; beta<3; beta++)
      {
        if (alpha == beta)
          continue;
        if (pa[alpha] == pa[beta])
	{
          ret++;  
          continue;
	}
/*        if (pa[alpha] == pa[beta])
	{
          fprintf(stderr, "\n\n Warning - tensor has 2 or 3 same eigenvalues\n");
          fprintf(stderr, " function : dpdir_da (file %s, line %d)\n", __FILE__, __LINE__);
          abort();
	}*/
        dpd_da[k][l] += (t[k][alpha]*t[l][beta]+t[k][beta]*t[l][alpha])*t[i][beta]/(pa[alpha]-pa[beta]);
      }
      dpd_da[k][l] /= 2.0; 
    } 
  }
  return ret;
}
