#include <string.h>
#include <stdlib.h>
#include "bocon.h"
#include "parser.h"



/**
  Constructor initializes data members
  Parameters :
    none
  Returns :
    nothing
*/
bocon::bocon()
{
  nn = 0L;
  ndir = 0L;
  dir = NULL;
  con = NULL;
  expr = NULL;
}



/**
  Method reads data from the text file specified by parameter in, data are
    in preprocessor format
  Parameters :
    @param in    - pointer to the opened text file, where the data will be read
    @param ndofn - number of dofs in given node with prescribed condition
  Returns :
    @retval 0 - on success
    @retval 1 - number of boundary conditions is not in range <1, ndofn>
    @retval 2 - direction of a boundary condition is not in range <1, ndofn>
*/
long bocon::read(XFILE *in, long ndofn)
{
  long nbc, td, i;
  double tc;
  Parser par;
  Equation *eq;

  // deallocate eventually used memory
  if (dir)
    delete [] dir;
  if (con)
    delete [] con;
  if (expr)
  {
    for (i = 0; i < ndir; i++)
      delete [] expr[i];
    delete [] expr;
  }

  ndir = ndofn;
  xfscanf (in, "%k%ld", "num_bc", &nbc);
  // reading number of conditions
  if ((nbc < 1) || (nbc > ndir))
  // which should be in range <1,ndofn>
    return(1);
  // allocating memory for arrays
  dir = new long[ndir];
  for (i = 0; i < ndir; dir[i] = -1, i++);
  // if dir[i] == -1 there is no codition in the i direction
  con = new double[ndir];
  memset(con, 0, sizeof(*con)*ndir);
  expr = new char*[ndir];
  memset(expr, 0, sizeof(*expr)*ndir);

  // loop which reads conditions in particular directions
  for (i = 0; i < nbc; i++)
  {
    xfscanf(in, "%k%ld", "dir", &td); // reading direction
    xfscanf(in, "%k%le", "cond", &tc); // reading condition
    if ((labs(td) < 1) || (labs(td) > ndir))
    // direction should be in range <1, ndir>
      return(2);
    if (td > 0)
    {
      td--;
      dir[td] = td;
      con[td] = tc;
    }
    if (td < 0)
    // if direction number is negative -> dynamic condition
    {
      td = labs(td);
      td--;
      dir[td] = td;
      con[td] = tc;
      expr[i] = new char[1025];
      expr[0] = '\0';
      xfscanf(in, "%1024s", expr[i]); // read string with expression
      eq = par.TextToTree(expr[i]); // expansion expression in the string
                                    // by parser to binary tree
      if (eq == NULL)
      // expansion failed
        return(3);
    }
  }
  return(0);
}



/**
  Method tries to merge two boundary conditions

  @param tbc  - object of boundary condition which will be copied

  @retval Method returns pointer to new object of boundary condition
*/
bocon *bocon::merge(bocon *tbc)
{
  bocon *ret = new bocon;
  long i, l;

  ret->nn = nn;
  ret->ndir = ndir;
  ret->dir = new long[ndir];
  for (i = 0; i < ndir; ret->dir[i] = -1, i++);
  ret->con = new double[ndir];
  memset(ret->con, 0, sizeof(*ret->con)*ndir);
  ret->expr = new char*[ndir];
  memset(ret->expr, 0, sizeof(*ret->expr)*ndir);
  for (i=0; i<ndir; i++)
  {
    // compare BC direction indicators
    if (dir[i] == tbc->dir[i])
    {      
      // both BC have prescribed condition in this direction
      ret->dir[i] = dir[i];
      // compare BC condition type
      if (con[i] == tbc->con[i])
      {
        // type of condition is the same
        ret->con[i] = con[i];
        if ((tbc->expr[i] == NULL) && (expr[i] == NULL))
        {
          // no BC for dynamic 
          ret->expr[i] = NULL;
          continue;
        }
        else
        {
          if (strcmp(expr[i], tbc->expr[i]) == 0)
          {
            // time functions for dynamic have to be indentical
            l = strlen(expr[i]);
            ret->expr[i] = new char[l+1];
            memcpy(ret->expr[i], expr[i], sizeof(*expr[i])*(l+1));
            continue;
	  }
          else
          // conflict in merged conditions
          {
            return NULL;
          }
	}
      }
    }
    else
    {
      // only one of the BC has been assigned to this direction
      if (dir[i] != -1)
      {
        // copy dir of this BC
        ret->dir[i] = dir[i];
        ret->con[i] = con[i];
        if (expr[i] == NULL)
          ret->expr[i] = NULL;
        else
        {
          l = strlen(expr[i]);
          ret->expr[i] = new char[l+1];
          memcpy(ret->expr[i], expr[i], sizeof(*expr[i])*(l+1));
          continue;
        }
      }
      else
      {
        // copy dir of tbc
        ret->dir[i] = tbc->dir[i];
        ret->con[i] = tbc->con[i];
        if (tbc->expr[i] == NULL)
          ret->expr[i] = NULL;
        else
        {
          l = strlen(tbc->expr[i]);
          ret->expr[i] = new char[l+1];
          memcpy(ret->expr[i], tbc->expr[i], sizeof(*tbc->expr[i])*(l+1));
          continue;
        }
      }        
    }
  }
  return ret;
}



/**
  Method creates copy of given object 

  Returns:
  Method returns pointer to newly alocated object of boundary condition.
*/
bocon *bocon::copy(void)
{
  bocon *ret = new bocon;
  long i, l;

  ret->nn = nn;
  ret->ndir = ndir;
  ret->dir = new long[ndir];
  memcpy(ret->dir, dir, sizeof(*dir)*ndir);
  ret->con = new double[ndir];
  memcpy(ret->con, con, sizeof(*con)*ndir);
  ret->expr = new char*[ndir];
  memset(ret->expr, 0, sizeof(*ret->expr)*ndir);
  for(i=0; i<ndir; i++)
  {
    if (expr[i])
    {
      l = strlen(expr[i]);
      ret->expr[i] = new char[l];
      memcpy(ret->expr[i], expr[i], sizeof(*expr[i])*l);
    }
  }
  return ret;
}



bocon::~bocon()
{
  delete [] dir;
  delete [] con;
  if (expr)
  {
    for (long i = 0; i < ndir; i++)
      delete [] expr[i];
    delete [] expr;
  }
}
