#include <math.h>
#include "loadcase.h"
#include "global.h"
#include "globmat.h"
#include "elemswitch.h"
#include "gtopology.h"
#include "matrix.h"
#include "vector.h"
#include "loadn.h"
#include "loadel.h"
#include "element.h"
#include "node.h"
#include "elemhead.h"



/**
  Constructor initializes data members to zero or default values.

  Created by JK,
*/
loadcase::loadcase (void)
{
  //  number of loaded nodes
  nln = 0;
  //  number of loaded elements
  nle = 0;
  //  number of prescribed displacements
  npd = 0;
  //  number of prescribed temperature changes
  npt = 0;
  tempchang=0;
  
  //  loaded nodes
  lon = NULL;
  //  loaded elements
  loe = NULL;
  //  prescribed displacements
  pd = NULL;
  //  prescribed temperature changes
  pt = NULL;
  // prescribed macrostresses for homogeniztaion problems (Mp->homog == 3)
  mstress = NULL;
}



/**
  Destructor releases allocated memory of the loadcase object.

  Created by JK,
*/
loadcase::~loadcase (void)
{
  delete [] lon;  delete [] loe;  delete [] pd;  delete [] pt; delete [] mstress;
}



/**
  Function reads characteristics of static load case from the opened 
  text file.

  @param in - pointer to the opened XFILE
   
  @return The function does not return anything.

  Created by JK, 24.7.2001
*/
void loadcase::read (XFILE *in)
{
  long i;
  
  //  loaded nodes
  xfscanf (in,"%ld",&nln);
  fprintf (stdout,"\n the number of loaded nodes %ld",nln);
  lon = new loadn [nln];
  for (i=0;i<nln;i++){
    lon[i].read (in);
  }
  
  //  loaded elements
  xfscanf (in,"%ld",&nle);
  fprintf (stdout,"\n the number of loaded elements %ld",nle);
  loe = new loadel [nle];
  for (i=0;i<nle;i++){
    loe[i].read (in);
  }
  
  //  prescribed displacements
  xfscanf (in,"%ld",&npd);
  fprintf (stdout,"\n the number of prescribed displacements %ld",npd);
  pd = new double [npd];
  for (i=0;i<npd;i++){
    xfscanf (in,"%lf",pd+i);
  }
  
  //  prescribed temperature changes
  xfscanf (in,"%ld",&tempchang);
  if (tempchang==0)
    fprintf (stdout,"\n no temperature changes are described");
  switch (tempchang)
  {
    case 0:
      break;
    case 1: // temperature is defined in MEFEL input file
    case 2: // scaled temperature is defined in MEFEL input file
      if (Mp->temperature < 2)
        Mp->temperature=1;
      else
      {
        print_err("type 1 or 2 of temperature load cannot be used together with type 3", __FILE__, __LINE__, __func__);
        abort();
      }
      break;
    case 3: // temperature is calculated by TRFEL
      if (Mp->temperature == 1)
      {
        print_err("type 1 or 2 of temperature load cannot be used together with type 3", __FILE__, __LINE__, __func__);
        abort();
      }
      else
        Mp->temperature=2;
      break;
    default:
      print_err("unknown type of temperature load (=%ld) is required", __FILE__, __LINE__, __func__, tempchang);
      abort();
  }
  
  if (tempchang==1 || tempchang==2){
    npt = Mt->nn;
    pt = new double [npt];
    for (i=0;i<npt;i++){
      xfscanf (in,"%lf",pt+i);
    }
    
  }
  if (Mp->homog == 3)
  {
    long ncomp = Mt->give_tncomp(0);
    mstress = new double [ncomp];
    memset(mstress, 0, sizeof(*mstress)*ncomp);
    for (i=0; i<ncomp; i++)
      xfscanf(in, "%lf", mstress+i);
  }
  
  //if (tempchang==3)
  //rhstemp = new double [Ndofm];
  
}


/**
  Function prints characteristics of static load case into the opened 
  text file.

  @param out - pointer to the opened FILE
   
  @return The function does not return anything.

  TKr, 07/02/2013 accroding to read (XFILE *in)
*/
void loadcase::print (FILE *out)
{
  long i;
  
  //  loaded nodes
  fprintf(out,"\n## loaded nodes:");
  fprintf (out,"\n%ld\n\n",nln);
  for (i=0;i<nln;i++){
    lon[i].print (out);
  }
  
  //  loaded elements
  fprintf(out,"\n## loaded elements:");
  fprintf (out,"\n%ld\n\n",nle);
  for (i=0;i<nle;i++){
    loe[i].print (out,0);
  }
  
  //  prescribed displacements
  fprintf(out,"\n##   prescribed displacements:");
  fprintf (out,"\n%ld\n\n",npd);
  for (i=0;i<npd;i++){
    fprintf (out,"\n %lf",pd[i]);
  }
  
  //  prescribed temperature changes
  fprintf(out,"\n##   temperature changes:");
  fprintf(out,"\n%ld\n\n",tempchang);
  
  if (tempchang==1 || tempchang==2){
    for (i=0;i<npt;i++){
      fprintf (out,"\n %lf",pt[i]);
    }
  }

  if (Mp->homog == 3)
  {
    long ncomp = Mt->give_tncomp(0);
    for (i=0; i<ncomp; i++)
      fprintf(out, "%lf ", mstress[i]);
    fprintf(out, "\n");
  }
}



/**
  Function assembles %vector of right hand side. 
  It is created by prescribed displacements, forces, moments, eigenstrains, etc.   
  In case that the flv is not NULL, %vector of load caused by forces only is stored in flv.

  @param lcid  - load case id
  @param rhs   - pointer to the right hand side
  @param flv   - array of load %vector caused by forces only (default value is NULL)
  @param scale - scale factor, it is used for subload cases
   
  Created by JK, 24.7.2001
  Modifed by Tomas Koudelka, 7.2008
*/
void loadcase::assemble (long lcid,double *rhs,double *flv,double scale)
{
  long i,j,k,eid,ndofn,ndofe,ndofemn,ne,ncomp;
  ivector cn;
  vector r,f,af,anf;
  matrix sm;
  
  //  array rhs and flv have to be cleaned in the caller (above) function
  
  // ***************************
  //  contributions from nodes
  // ***************************
  for (i=0;i<nln;i++){
    ndofn=Mt->give_ndofn (lon[i].nid);
    if (ndofn<0){
      print_err("loaded node %ld is a hanging node",__FILE__,__LINE__,__func__,i++);
    }
    for (j=0;j<ndofn;j++){
      k=Mt->give_dof (lon[i].nid,j);
      if (k<=0)  continue;
      if (k>0)  rhs[k-1]+=lon[i].f[j];
    }
  }
  
  // ******************************
  //  contributions from elements
  // ******************************
  //  loop over the loaded elements
  for (i=0;i<nle;i++){
    //  element id
    eid=loe[i].eid;
    if (Gtm->leso[eid]==1){
      
      //  the number of DOFs on element
      ndofe=Mt->give_ndofe (eid);
      //  the number of DOFs on element
      //  this number is equal to the number of DOFs with master nodes generated by hanging nodes
      ndofemn = Gtm->give_ndofe (eid);
      
      reallocv (ndofemn,cn);
      //  code numbers on element
      Mt->give_code_numbers (eid,cn.a);
      
      if (ndofe != ndofemn){
	//  the element contains hanging node
	//  the vector has to be transformed
	reallocv (ndofemn,anf);
	mtxv (*Mt->elements[i].tmat,loe[i].nf,anf);
	locglob (rhs,anf.a,cn.a,ndofemn);
      }else{
	//  the element does not contain hanging node
	locglob (rhs,loe[i].nf.a,cn.a,ndofe);
      }
    }
  }
  
  cmulv(scale,rhs,Ndofm);
  // assembling of load vector caused by forces only    
  if (flv)
    copyv (rhs,flv,Ndofm);
  
  // **********************************************
  //  contributions from prescribed displacements
  // **********************************************
  if ((npd>0) && (Mp->tprob != mat_nonlinear_statics)){
    //  the number of elements
    ne = Mt->ne;
    for (i=0;i<ne;i++){
      if (Mt->elements[i].prescdispl==1){
	//  only elements with prescribed displacements are taken into account
	
	//  the number of DOFs on element
	//  this number is equal to the number of DOFs without hanging nodes
	ndofe=Mt->give_ndofe (i);
	reallocm (ndofe,ndofe,sm);
	//  stiffness matrix of an element
	stiffmat (lcid,i,sm);
	
	reallocv (ndofe,r);
	reallocv (ndofe,f);
	//  prescribed displacements on element
	elprdispl (lcid,i,r.a);
	//  Kr=f
	mxv (sm,r,f);
	cmulv (-1.0,f);
	
	//  the number of DOFs on element
	//  this number is equal to the number of DOFs with master nodes generated by hanging nodes
	ndofemn = Gtm->give_ndofe (i);
	reallocv (ndofemn,cn);
	
	//  code numbers on element
	Mt->give_code_numbers (i,cn.a);
	
	if (ndofe != ndofemn){
	  //  the element contains hanging node
	  //  the vector f has to be transformed
	  reallocv (ndofemn,af);
	  mtxv (*Mt->elements[i].tmat,f,af);
	  locglob (rhs,af.a,cn.a,ndofemn);
	}else{
	  //  the element does not contain hanging node
	  locglob (rhs,f.a,cn.a,ndofe);
	}
	
      }
    }
  }

  //  contributions from temperature changes
  tempercontrib (lcid,rhs,scale);

  // contrubution from macro stresses in the homogenization problem
  if (Mp->homog == 3)
  {
    ncomp = Mt->give_tncomp(0);
    j=0;
    for (i=Ndofm-ncomp; i<Ndofm; i++)
    {
      rhs[i] += mstress[j];
      j++;
    }
  }
}



/**
  Function computes contributions from temperature changes
  to the right hand side (%vector of nodal forces).
   
  @param lcid - load case id
  @param rhs - pointer to the right hand side
  @param scale - scale factor, it is used for subload cases
   
  Created by JK, Tomas Koudelka, 21.11.2007
*/
void loadcase::tempercontrib (long lcid,double *rhs,double scale)
{
  //  contributions from temperature changes
  if (tempchang==1){
    //  contribution to the time-independent problems from temperatures
    //  read from input files
    
    //  type of eigenstrain is temperature strain
    Mm->est=tempstrain;

    //  approximation of nodal temperatures to integration points
    intpointval (pt,temperature,1.0);
    
    //  thermal material models compute temperature strains
    Mm->temprstrains (lcid);
    
    //  computation of nodal forces caused by temperature
    nodal_eigstrain_forces (lcid,rhs,Mp->time);
  }
  
  if (tempchang==2){
    //  contribution to the time-dependent problems from temperatures
    //  read from input file, temperature is changed by multiplication of
    //  read function
    
    //  type of eigenstrain is temperature strain
    Mm->est=tempstrain;

    //  approximation of nodal temperatures
    intpointval (pt,temperature,scale);
    
    //  thermal material models compute temperature strains
    Mm->temprstrains (lcid);
    
    nodal_eigstrain_forces (lcid,rhs,Mp->time);
  }

  if (tempchang==3){
    //  contribution to the time-dependent coupled problems from
    //  temperatures obtained from heat transfer (from TRFEL)
    
    //  type of eigenstrain is temperature strain
    Mm->est=tempstrain;

    //  thermal material models compute temperature strains
    Mm->temprstrains (lcid);
    
    //  computation of nodal forces caused by temperature
    nodal_eigstrain_forces (lcid,rhs,Mp->time);
  }
  
}



/**
   Function computes reactions.
   
   @param lcid - load case id
   
   @return The function does not return anything.
   
   Created by JK,
*/
void loadcase::compute_reactions (long lcid)
{
  long i,j,k,ii,nne,ndofn,nn,ne,ndofe,ndofemn,eid;
  ivector nod;
  vector f,tf;
  
  //  the number of nodes
  nn=Mt->nn;
  //  loop over the number of nodes
  for (i=0;i<nn;i++){
    if (Mt->nodes[i].react==1){
      //  only nodes with constraint/reaction are taken into account
      
      //  the number of DOFs in the i-th node
      ndofn=Mt->give_ndofn (i);
      for (k=0;k<ndofn;k++){
	Mt->nodes[i].r[k]=0.0;
      }
    }
  }
  
  //  the number of elements
  ne=Mt->ne;
  //  loop over the number of elements
  for (i=0;i<ne;i++){
    if ((Mt->elements[i].react==1) && (Gtm->leso[i]==1)){
      //  only elements connected to a node with reaction are taken into account
      
      //  the number of DOFs on element
      //  this number is generated by elements without hanging nodes
      ndofe=Mt->give_ndofe (i);
      reallocv (ndofe,f);
      //  nodal forces are computed
      elem_internal_forces (i,lcid,f);
      
      //  the number of DOFs on element
      //  this number is equal to the number of DOFs with master nodes generated by hanging nodes
      ndofemn = Gtm->give_ndofe (i);
      
      //  the number of nodes on element
      //  if there are hanging nodes, the variable will be rewritten
      nne=Mt->give_nne (i);
      
      if (ndofe != ndofemn){
	//  the element contains hanging node
	//  the vector of internal forces has to be transformed
	
	vector af;
	reallocv (ndofemn,af);
	//  transformation of the vector f
	mtxv (*Mt->elements[i].tmat,f,af);
	reallocv (ndofemn,f);
	copyv (af,f);
	
	//  the number of nodes on element
	//  it is equal to the number of master nodes
	nne=Gtm->gelements[i].nmne;
	reallocv (nne,nod);
	Gtm->gelements[i].give_master_nodes (nod);
	
      }//  end of the statement if (ndofe != ndofemn){
      
      if (ndofe == ndofemn){
	reallocv (nne,nod);
	Mt->give_elemnodes (i,nod);
      }
      //  at this moment, the arrays nod and f contain the appropriate number
      //  of components
      
      ii=0;
      //  loop over the number of nodes on element
      //  in the case of element with hanging nodes, the correct number of nodes is used
      for (j=0;j<nne;j++){
	
	//  the number of DOFs in node
	ndofn=Mt->give_ndofn (nod(j));
	if (Mt->nodes[nod(j)].react==1){
	  //  only nodes with constraint/reaction are taken into account
	  
	  //  loop over the number of DOFs in the actual node
	  for (k=0;k<ndofn;k++){
	    Mt->nodes[nod(j)].r[k]+=f[ii];
	    ii++;
	  }//  end of the loop over the number of DOFs in the actual node
	}
	else  ii+=ndofn;
      }//  end of the loop over the number of nodes on element
      
    }
  }//  end of the loop over the number of elements
  
  
  //  loop over the number of loaded elements
  for (i=0;i<nle;i++){
    //  element id
    eid=loe[i].eid;
    
    if ((Mt->elements[eid].react==1) && (Gtm->leso[eid]==1)){
      //  only elements with reactions are taken into account
      
      //  the number of DOFs on element
      ndofe=Mt->give_ndofe (eid);
      //  the number of DOFs on element
      //  this number is equal to the number of DOFs with master nodes generated by hanging nodes
      ndofemn = Gtm->give_ndofe (eid);

      if (ndofe != ndofemn){
	//  the element contains hanging node
	//  the vector of internal forces has to be transformed
	
	reallocv (ndofemn,f);
	//  transformation of the vector f
	mtxv (*Mt->elements[eid].tmat,loe[i].nf,f);
	
	//  the number of nodes on element
	//  it is equal to the number of master nodes
	nne=Gtm->gelements[eid].nmne;
	reallocv (nne,nod);
	Gtm->gelements[eid].give_master_nodes (nod);
	
      }//  end of the statement if (ndofe != ndofemn){
      
      if (ndofe == ndofemn){
	//  the number of nodes on element
	nne=Mt->give_nne (eid);
	reallocv (nne,nod);
	Mt->give_elemnodes (eid,nod);
	reallocv (ndofe,f);
	copyv (loe[i].nf,f);
      }
      //  at this moment, the arrays nod and f contain the appropriate number
      //  of components
      
      ii=0;
      //  loop over the number of nodes on element
      //  if hanging nodes are present, the loop takes them into account
      for (j=0;j<nne;j++){
	
	//  the number of DOFs in node
	ndofn=Mt->give_ndofn (nod(j));
	if (Mt->nodes[nod(j)].react==1){
	  //  only nodes where reaction is are taken into account
	  
	  for (k=0;k<ndofn;k++){
	    Mt->nodes[nod(j)].r[k]-=f[ii];
	    ii++;
	  }
	}
	else  ii+=ndofn;
      }
    }
  }
  
  
  
  /*  if (Mp->eigstrains==1){
    
    for (i=0;i<ne;i++){
	if ((Mt->elements[i].react==1) && (Gtm->leso[i]==1)){
	te = Mt->give_elem_type (i);
	ndofe=Mt->give_ndofe (i);
	allocv (ndofe,f);
	
	nne=Mt->give_nne (i);
	allocv (nne,nod);
	Mt->give_elemnodes (i,nod);
        //  type of eigenstrain is eigenstrain
        Mm->est=eigstrain;
	elem_eigstrain_forces (lcid,i,f);
	ii=0;
	for (j=0;j<nne;j++){
	  ndofn=Mt->give_ndofn (nod(j));
	  if (Mt->nodes[nod(j)].react==1){
	    for (k=0;k<ndofn;k++){
	      Mt->nodes[nod(j)].r[k]-=f[ii];
	      ii++;
	    }
	  }
	  else  ii+=ndofn;
	}
	destrv(nod);
	destrv (f);
      }
    }
  }
  */
  
  /*
  //  sum of all reactions in appropriate direction
  double totr=0.0;
  for (i=0;i<nn;i++){
    if (Mt->nodes[i].react==1){
      ndofn=Mt->give_ndofn (i);
      cn = new long [ndofn];
      Mt->give_node_code_numbers (i,cn);
      for (k=0;k<ndofn;k++){
	if (cn[k]==-1)
	  totr+=Mt->nodes[i].r[k];
      }
      delete [] cn;
    }
  }
  

  FILE *vysl;
  vysl = fopen ("reakce.txt","a");
  fprintf (Outdm->outf,"\n\n\n total reaction %15.12le\n\n",totr);
  fprintf (vysl," %15.12le\n",totr);
  fclose (vysl);
  */

}

