#include <string.h>
#include "mtsolver.h"
#include "backupsol.h"
#include "mtglvec.h"
#include "global.h"
#include "globmat.h"
#include "mechprint.h"
#include "matrix.h"
#include "vector.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "dloadcase.h"
#include "elemswitch.h"
#include "nssolver.h"
#include "newtonraph.h"

#ifndef FNAMELEN
 #define FNAMELEN 1001
#endif



/**
  The function calls solver of time dependent problems (viscoplasticity)
  for each load case.

  @return The function does not return anything.

  Created by JK,
*/
void solve_time_dep_prob ()
{
  long lcid;
  
  //  load case must be equal to zero, no superposition can be used
  //  in this type of analysis
  lcid=0;
  
  //  solver of the problem
  visco_solver2 (lcid);
}



/**
  Function solves time dependent problem (viscoplasticity)
   
  @param lcid - load case id
   
  @return The function does not return anything.
   
  Created by JK, 27.10.2001
  Modified by Tomas Koudelka
*/
void visco_solver (long lcid)
{
  long i,j,k,n,ini,nsts;
  double dt,dtmin,dtdef,end_time;
  double *f,*fl,*fi,*fb,*fp,*r,*dr,*lhsb;
  double norf, norfa, err, zero;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);
  Mp->nlman->divc_step = new double[10];
  Mp->nlman->divc_err  = new double[10];
  
  //  initialization phase

  //  maximum number of iterations in inner loop
  ini = Mp->nlman->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->nlman->errnr;
  //  number of mechanical degrees of freedom
  n=Ndofm;
  //  computer zero
  zero = Mp->zero;

  //  vector of nodal displacements
  r = Lsrs->give_lhs (lcid);
  nullv (r,n);
  //  vector of nodal prescribed forces
  f = Lsrs->give_rhs (lcid);

  // allocation of vectors  

  //  vector of increments of nodal displacements
  dr   = new double [n];
  nullv (dr,n);
  //  vector of prescribed forces from the previous step
  fp   = new double [n];
  nullv (fp,n);
  //  vector of prescribed force loads, it does not contain forces caused by temperature, etc.
  fl   = new double [n];
  nullv (fl,n);
  //  vector of internal forces
  fi   = new double [n];
  nullv (fi,n);
  //  auxiliary force vector
  fb   = new double [n];
  nullv (fb,n);
  //  backup of the nodal displacements
  lhsb = new double [n];
  nullv (lhsb,n);
  
  
  //  initial time
  Mp->time=Mp->timecon.starttime ();
  //  initial time increment
  dt = Mp->timecon.initialtimeincr ();
  Mp->dtime = dt;
  //  end time
  end_time = Mp->timecon.endtime ();
  //  minimum time increment
  dtmin=Mp->timecon.dtmin;

  //  number of step
  i=0;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;

  //  inital printing of output and graphical informations
  if (Mp->hdbcont.restore_stat()){
    if (Mespr==1)
      fprintf (stdout,"\n Reading of backup file\n");
    solver_restore (r,fp,i,Mp->time,dt,&Mp->timecon,n);
    //  initiation of mechanical material models
    Mm->initmaterialmodels(lcid);
    print_init(-1, "at");
    //print_step(lcid,i,Mp->time,f);
    //print_flush();
  }
  else
  {
    //  initiation of mechanical material models
    Mm->initmaterialmodels(lcid);
    print_init(-1, "wt");
    print_step(lcid, i, Mp->time, f);
    //print_step(lcid, i, 0.0, f);
    print_flush();
  }
  //print_close();

  // ***************************
  //   main iteration loop  ****
  // ***************************
  do{
    //new time increment
    Mp->time = Mp->timecon.newtime (dt);
    Mp->dtime = dt;
    //  new step number
    i++;
    Mp->istep = i;
    Mp->jstep = -1;
    //  indicator of strain computation
    //  it means, strains at integration points have not been computed
    Mp->strainstate=0;
    //  indicator of stress computation
    //  it means, stresses at integration points have not been computed
    Mp->stressstate=0;
    //  indicator of computation of other array
    //  it means, stresses at integration points have not been computed
    Mp->otherstate=0;

    //  backup of attained nodal values
    for (j=0;j<n;j++){
      lhsb[j]=r[j];
    }

    if (Mespr==1)  fprintf (stdout,"\n\n -------------------------------------------------------------");
    if (Mespr==1)  fprintf (stdout,"\n Time step = %ld,  Time %e,  Time increment = %e",i,Mp->time,dt);
    if (Mespr==1)  fprintf (stdout,"\n -------------------------------------------------------------\n");

    //  assembling of vectors of prescribed nodal forces
    mefel_right_hand_side (lcid,f,fl);

    //  computation of sums of force components in particular directions
    //  it is used in special creep and consolidation models
    Mb->comp_sum (f);
    //fprintf(Out, "\n# i=%ld, time=%le[s], time=%le[d], fx=%le, fy=%le, fz=%le", 
    //      i, Mp->time, Mp->time/86400, Mb->sumcomp[0], Mb->sumcomp[1], Mb->sumcomp[2]);
    //fflush(Out);

    //  increments of prescribed forces
    incr_internal_forces (lcid,fi);

    //  right hand side assembling
    for (j=0;j<n;j++){
      fb[j]=f[j]-fp[j]+fi[j];
    }
    
/*    norf = ss(f,f,nm);
    fprintf(stdout, "\n\n norm(f)=%le,", norf);
    norf = ss(fp,fp,nm);
    fprintf(stdout, " norm(fp)=%le,", norf);
    norf = ss(fi,fi,nm);
    fprintf(stdout, " norm(fi)=%le,", norf);
    norf = ss(fb,fb,nm);
    fprintf(stdout, " norm(f-fp+fi)=%le\n\n", norf); */

    //  assembling of tangent stiffness matrix
    if ((Mp->nlman->stmat==tangent_stiff) || (i == 1) || (Smat==NULL))
      stiffness_matrix (lcid);

    //  solution of K(r).v=F
    Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

    //  new displacement vector
    for (j=0;j<n;j++){
      r[j]+=dr[j];
    }

    //  computation of internal forces
    internal_forces (lcid,fi);

    // norm of total load vector      
    norfa=ss(f,f,n);
    // vector of unbalanced forces
    for (j=0;j<n;j++){
      fb[j] = fl[j] - fi[j];
    }

    //  norm of vector of unbalanced forces
    norf=ss(fb,fb,n);
    if (norfa > 0.0)
      norf /= norfa;

    if (Mespr==1)  fprintf (stdout,"\n\n Norf before inner iteration loop norf = %e\n\n\n",norf);
    j = 0;
    if (norf<err){
      nsts++;      
    }
    else{
      // iteration of unbalanced forces caused by time independent models
      //  internal iteration loop
      fillm(0.0, lsm_a);
      fillv(0.0, lsm_r);
      for (j=0;j<ini;j++)
      {
        Mp->jstep = j;
        //assembling of stiffness matrix for fullnewton
        if ((Mp->nlman->stmat==tangent_stiff) && (j%5 == 0))
          stiffness_matrix (lcid);
        //  solution of K(r).v=F
        Mp->ssle->solve_system (Gtm,Smat,dr,fb,Out);

        //  new displacement vector
        for (k=0;k<n;k++)
	  r[k]+=dr[k];

        //  computation of internal forces
        internal_forces (lcid,fi);

        //  vector of unbalanced forces
        for (k=0;k<n;k++)
          fb[k]=fl[k] - fi[k];

        //  norm of vector of unbalanced forces
        norf=ss(fb,fb,n);
        if (norfa > 0.0)
          norf /= norfa;

//        if ((Mespr==1)&&(j%10==0)) // print each 10-th step
	if (Mespr==1)
  	  fprintf (stdout,"\n Inner loop number j = %ld,     norf = %e \n",j,norf);

        if (norf<err)  break; // convergence attained

        // divergence detection with help of least square method
        if (j > 10)
        {
          if (lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), Mp->nlman->divc_step[j%10], 
                       Mp->nlman->divc_err[j%10], norf, Mp->zero,1) > 0.0)
	  {
	    fprintf (stdout,"\n\n Divergence of inner iteation loop is detected. Inner loop is stopped.\n\n");
	    break;
	  }
          Mp->nlman->divc_step[j%10] = double(j+1);
          Mp->nlman->divc_err[j%10] = err;
	}
        else
        {
          lsm_quad(lsm_a, lsm_r, lsm_l, double(j+1), norf, 0.0, 0.0, Mp->zero,0);
          Mp->nlman->divc_step[j%10] = double(j+1);
          Mp->nlman->divc_err[j%10] = err;
        }
      }
    }

    if (j==ini || norf>err)
    {
      // equilibrium state was not attained
      if (Mespr==1) fprintf (stdout,"\n\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);

      //  backup is used for actual values
      for (j=0;j<n;j++)
        r[j]=lhsb[j];

      //  reduction of the time increment because
      //  inner loop was not able to enforce equilibrium
      dt/=2.0;
      Mp->timecon.oldtime ();
      i--;

      if (Mespr==1)  fprintf (stdout,"\n\n time increment is reduced because the inner loop was not able to enforce equilibrium");
      if (dt<dtmin)
      {
        if (Mespr==1)  fprintf (stderr,"\n\n time increment is less than minimum time increment");
        if (Mespr==1)  fprintf (stderr,"\n computation failed (file %s, line %d)\n",__FILE__,__LINE__);
        if (Mespr==1)  fprintf (stderr,"\n FORCED output of results from this step is performed\n");
        compute_req_val (lcid);
	print_step_forced(lcid, i, Mp->time, fb);
        //print_step_forced(lcid, i, Mb->dlc[0].gf[0].getval(Mp->time), fb);
        print_flush();
        break;
      }
    }
    else
    {
      // equilibrium state was attained
      if (Mespr==1)  fprintf (stdout,"\n\n Last inner iteration step number j = %ld,     norf = %e \n\n",j,norf);

      //  actual total load vector becomes previous total load vector
      for (j=0;j<n;j++)
        fp[j]=f[j];

      dtdef = Mp->timecon.actualforwtimeincr ();
      if (nsts==2)
      {
        dt*=2.0;
        nsts=0;
        if (Mespr==1)  fprintf (stdout,"\n\n time increment is enlarged to %le because no inner loop was neccessary in previous 3 steps", dt);
      }
//      if (dt>dtdef)
//        dt=dtdef;

      // print resulting vector of internal forces
      Mb->comp_sum (fi);
      //fprintf(Out, "\n# i=%ld, time=%le[s], time=%le[d], fix=%le, fiy=%le, fiz=%le", 
      //      i, Mp->time, Mp->time/86400, Mb->sumcomp[0], Mb->sumcomp[1], Mb->sumcomp[2]);
      //fflush(Out);

      //  update of values stored at integration points
      Mm->updateipval();
      compute_req_val (lcid);
      print_step(lcid, i, Mp->time, fi);
      //print_step(lcid, i, Mb->dlc[0].gf[0].getval(Mp->time), fl);
      print_flush();
	
      if ((Mp->timecon.isitimptime()==1) && (Mp->hdbcont.save_stat()))
      {
        if (Mespr==1)
	  fprintf (stdout,"\n Creating backup file\n");
        solver_save (r,fp,i,Mp->time,dt,&Mp->timecon,n);
      }
    }
  }while(Mp->time<=end_time);
  
  print_close ();
  
  delete [] fi;  
  delete [] fb;  
  delete [] fp;  
  delete [] dr; 
  delete [] fl;  
  delete [] lhsb;
}



/**
  Function solves time dependent problem (viscoplasticity).
  It uses one_step concept instead of standard one.
  
  @param lcid - load case id
   
  @return The function does not return anything.
  
  Created by Tomas Koudelka, 11.2011, corrected by TKr 27/03/2013
*/
void visco_solver2 (long lcid)
{
  long i, li, nsts, ret;
  double time, dt, dtmin, dtmax, dtr, end_time;
  mt_glob_vec mt_gv;
  
  //
  //  initialization phase
  //
  visco_solver_init(lcid, mt_gv);
  
  //  initial time increment
  dt = Mp->timecon.initialtimeincr ();
  //  minimum time increment
  dtmin=Mp->timecon.dtmin;
  //  maximum time increment
  dtmax=Mp->timecon.dtmax;
  //  end time
  end_time = Mp->timecon.endtime ();
  
  //  number of step
  li = i = mt_gv.istep;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;
  // return status from the one_step function
  ret = 0;
  
  /*
  // test MuPiF
  vector tt(Mt->nn);
  fillv(10.0, tt);
  intpointval(tt.a, temperature, 1.0);
  fillv(0.0, tt);
  intpointval(tt.a, initial_temperature, 1.0);
  
  ret = one_step(lcid, 1.0, dt, i, mt_gv);
  */
  
  // ***************************
  //   main iteration loop  ****
  // ***************************
  do
    {
      //  new step number
      i++;
      
     // perform one time step
      time=Mp->timecon.newtime(dt);
      Mp->dtime = dt;
      ret = one_step(lcid, time, dt, dtr, i, li, mt_gv);
      
      if (ret >= 0) // equilibrium was attained
	{
	  if (ret == 0)
	    nsts++;      
	  if (nsts==2)
	    {
	      dt*=2.0;
	      nsts=0;
	      if (Mespr==1)  
		fprintf (stdout,"\n\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	    }
	  
	  if (dt>dtmax)//maximum time increment
	    dt = dtmax;
	}
      else
	{
	  //  reduction of the time increment because
	  //  inner loop was not able to enforce equilibrium
          if (dtr < 1.0)
            dt *= dtr;
          else
            dt/=2.0;
	  Mp->timecon.oldtime ();
	  i--;
	  
	  if (Mespr==1)  
	    fprintf (stdout,"\n\n time increment is reduced because the inner loop was not able to enforce equilibrium");
	  if (dt<dtmin)
	    {
	      if (Mespr==1)  fprintf (stderr,"\n\n time increment is less than minimum time increment");
	      if (Mespr==1)  fprintf (stderr,"\n computation failed (file %s, line %d)\n",__FILE__,__LINE__);
	      if (Mespr==1)  fprintf (stderr,"\n FORCED output of results from this step is performed\n");
	      compute_req_val (lcid);
	      print_step_forced(lcid, i, Mp->time, mt_gv.fb);
	      //print_step_forced(lcid, i, Mb->dlc[0].gf[0].getval(Mp->time), fb);
	      print_flush();
	      break;
	    }
	}
    }while(Mp->time<end_time);

  print_close ();
}



/**
  Function allocates and initializes global vectors used in mtsolver
  It is used in one_step concept solver.

  Created by Tomas Koudelka, 11.2011
*/
void visco_solver_init(long lcid, mt_glob_vec &mt_gv)
{
  long n;
  double dt;  
  //  number of mechanical degrees of freedom
  n=Ndofm;

  mt_gv.alloc(n);
  //  vector of nodal displacements
  mt_gv.r = Lsrs->give_lhs (lcid);
  //  vector of nodal prescribed forces
  mt_gv.f = Lsrs->give_rhs (lcid);

  //  initial time
  Mp->time=Mp->timecon.starttime ();

  //  inital printing of output and graphical informations
  if (Mp->hdbcont.restore_stat())
  {
    if (Mespr==1)
      fprintf (stdout,"\n Reading of backup file\n");
    solver_restore (mt_gv.r, mt_gv.fp, mt_gv.istep, Mp->time, dt, &Mp->timecon, n);
    //  initiation of mechanical material models
    Mm->initmaterialmodels(lcid);
    print_init(-1, "at");
    //print_step(lcid,i,Mp->time,f);
    //print_flush();
  }
  else
  {
    mt_gv.istep=0;
    //  initiation of mechanical material models
    Mm->initmaterialmodels(lcid);
    // compute stresses from eigenstrains or initial irreversible strains
    if (Mp->eigstrains > 0)
      internal_forces(lcid, mt_gv.fp);
    // print initial step to the output files
    print_init(-1, "wt");
    print_step(lcid, mt_gv.istep, Mp->time, mt_gv.f);
    //print_step(lcid, i, 0.0, f);
    print_flush();
  }
}



/**
  Function executes one step of time dependent mechanical algorithm
  (visco-elasticity, visco-plasticity, etc.)
   
  @param lcid - load case id
  @param time - actual time
  @param dt   - actual time increment
  @param dtr   - the minimum ratio of required time step size to the actual one required by material models (output)
  @param istep - time step id
  @param li    - time step id of the first performed time step
  @param mt_gv - structure with pointers to %vectors of righthand and lefthand side

  @retval  0 - if the equilibrium was attained with NO inner loop
  @retval  >0 - if the equilibrium was attained and inner loop WAS performed
  @retval -1 - if the equilibrium was NOT attained
   
  Created by JK, 19.9.2004
  Modified by TKo, 11.2011
*/
long one_step (long lcid,double time, double dt, double &dtr, long istep, long li, mt_glob_vec &mt_gv)
{
  long j,n,ini;
  double *f,*fl,*fi,*fb,*fp,*r,*dr,*lhsb;
  double norf, err;
  matrix lsm_a(3,3);
  vector lsm_r(3), lsm_l(3);
  Mp->nlman->divc_step = new double[10];
  Mp->nlman->divc_err  = new double[10];

  //new time increment
  Mp->time = time;
  //  new step number
  Mp->istep = istep;
  Mp->jstep = -1;
  //  indicator of strain computation
  //  it means, strains at integration points have not been computed
  Mp->strainstate=0;
  //  indicator of stress computation
  //  it means, stresses at integration points have not been computed
  Mp->stressstate=0;
  //  indicator of computation of other array
  //  it means, stresses at integration points have not been computed
  Mp->otherstate=0;

  //  vector of nodal displacements
  r  = mt_gv.r;
  //  vector of nodal prescribed forces
  f  = mt_gv.f;
  //  vector of increments of nodal displacements
  dr = mt_gv.dr;
  //  vector of prescribed forces from the previous step
  fp = mt_gv.fp;
  //  vector of prescribed force loads, it does not contain forces caused by temperature, etc.
  fl = mt_gv.fl;
  //  vector of internal forces
  fi = mt_gv.fi;
  //  auxiliary force vector
  fb = mt_gv.fb;
  //  backup of the nodal displacements
  lhsb = mt_gv.lhsb;

  //  number of mechanical degrees of freedom
  n=Ndofm;

  //  maximum number of iterations in inner loop
  ini = Mp->nlman->niilnr;
  //  required norm of vector of unbalanced forces
  err = Mp->nlman->errnr;

  //  backup of attained nodal values
  for (j=0;j<n;j++)
    lhsb[j]=r[j];
 

  if (Mespr==1)  fprintf (stdout,"\n\n ------------------------------------------------------------------------");
  if (Mespr==1)  fprintf (stdout,"\n Time step = %ld,  Time %e,  Time increment = %e",istep,Mp->time,dt);
  if (Mespr==1)  fprintf (stdout,"\n ------------------------------------------------------------------------\n");

  if (Mespr==1)  fprintf (Out,"\n\n ------------------------------------------------------------------------");
  if (Mespr==1)  fprintf (Out,"\n Time step = %ld,  Time %e,  Time increment = %e",istep,Mp->time,dt);
  if (Mespr==1)  fprintf (Out,"\n ------------------------------------------------------------------------\n");

  //  assembling of vectors of prescribed nodal forces
  mefel_right_hand_side (lcid,f,fl);
  
  //  computation of sums of force components in particular directions
  //  it is used in special creep and consolidation models
  Mb->comp_sum (f);
  //fprintf(Out, "\n# i=%ld, time=%le[s], time=%le[d], fx=%le, fy=%le, fz=%le", 
  //      istep, Mp->time, Mp->time/86400, Mb->sumcomp[0], Mb->sumcomp[1], Mb->sumcomp[2]);
  //fflush(Out);

  //  increments of prescribed forces
  incr_internal_forces (lcid,fi);

  // assembling of right hand side
  // fb[j]=f[j]-fp[j]+fi[j]
  subv(f, fp, fb, n);
  addv(fb, fi, fb, n);

  j = 0;  
  norf = gnewton_raphson_one_step(lcid, Mp->nlman, fl, r, fb, dr, fi, dtr, istep-1, j, li);

  if (norf>err)
  {
    // equilibrium state was not attained
    if (Mespr==1) fprintf (stdout,"\n\n Iteration process does not converge req. err = %e, reached err = %e", err,norf);
    
    //  backup is used for actual values
    //  r[k]=lhsb[k]
    copyv(lhsb, r, n);
    
    return -1;
  }

  // equilibrium state was attained
  if ((Mespr==1) && (j > 0))  fprintf (stdout,"\n\n Last inner iteration step number j = %ld,     norf = %e \n\n",j,norf);
  
  // actual total load vector becomes previous total load vector
  // fp[k]=f[k]
  copyv(f, fp, n);

  // print resulting vector of internal forces
  Mb->comp_sum (fi);
  //fprintf(Out, "\n# i=%ld, time=%le[s], time=%le[d], fix=%le, fiy=%le, fiz=%le", 
  //      istep, Mp->time, Mp->time/86400, Mb->sumcomp[0], Mb->sumcomp[1], Mb->sumcomp[2]);
  //fflush(Out);

  //  update of values stored at integration points
  Mm->updateipval();
  compute_req_val (lcid);
  print_step(lcid, istep, Mp->time, fl);
  print_flush();
	
  if ((Mp->timecon.isitimptime()==1) && (Mp->hdbcont.save_stat()))
  {
    if (Mespr==1)
      fprintf (stdout,"\n Creating backup file\n");
    solver_save (r,fp,istep,Mp->time,dt,&Mp->timecon,n);
  }
/*
  if (j == 0)
    return 0;

  return 1;
*/
  if (j < 0)
    return 0;

  return j;
}
