#include "npglvec.h"
#include "npsolvert.h"
#include "globalt.h"
#include "globmatt.h"
#include "elemswitcht.h"
#include "transprint.h"
#include "saddpoint.h"
#include "backupsolt.h"
#include <string.h>
#include <errno.h>

/**
   function solves nonstationary transport problem
   
   function selects a suitable numerical scheme for solution
   
   JK
*/
void solve_nonstationary_problem ()
{
  if (Tp->tprob == discont_nonstat_problem){
    //  formulation based on nodal unknowns is used
    linear_nonstat_solv_dform ();
  }
  else{
    //  formulation based on unknown first nodal derivatives
    //  with respect to time is used
    nonstat_solv_vform_comp ();
    //linear_nonstat_solv_vform ();
    //linear_nonstat_solv_dform ();
    //linear_nonstat_radiation_solv_dform ();
  }
}

/**
   function solves linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   according to JK, 20.12.2002 and TKo mtsolver.cpp
   revised 05/11/2012 by TKr
*/
void nonstat_solv_vform_comp ()
{
  long i,k,li,nsts,ret,n;
  double newtime,dt,dtmin,dtmax,dtdef,end_time;
  double *lhsb,*tdlhsb;
  np_glob_vec np_gv;
  long lcid = 0;

  //
  //  initialization phase
  //
  //  nodes - integration points interpolation
  approximation();  
  nonstat_solver_init(lcid, np_gv);

  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  //  minimum time increment
  dtmin=Tp->timecont.dtmin;
  //  maximum time increment
  dtmax=Tp->timecont.dtmax;
  //  end time
  end_time = Tp->timecont.endtime ();

  //  number of step
  li = i = np_gv.istep;
  //  number of successful time steps (number of steps without inner iterations)
  nsts=0;
  // return status from the one_step function
  ret = 0;

  // only for adaptive time increment
  if(Tp->timecont.tct > 0){
    n=Ndoft;
    lhsb   = new double [n];
    tdlhsb   = new double [n];
    nullv (lhsb,n);
    nullv (tdlhsb,n);
  }

  bool breakloop = false;
  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    i++;
  
    newtime = Tp->time = Tp->timecont.newtime(dt);
    //  computation of backward time step - only for adaptive time controler or nonconstant time increment
    if((Tp->timecont.timefun.tfunc != stat) && Tp->timecont.tct == 0)
      dt = Tp->timecont.actualbacktimeincr ();

    if(Tp->tprob == nonstationary_problem)
      // linear algorithm
      // perform one time step with linear solver
      ret = one_step_linear(lcid, newtime, dt, i, li, np_gv);
    
    if(Tp->tprob == nonlinear_nonstationary_problem){
      // non-linear algorithm
      // perform one time step with non-linear solver
      ret = one_step_nonlinear(lcid, newtime, dt, i, li, np_gv);
    }
    
    // handling with time controler for nonlinear solver, only for adaptive time increment
    if(Tp->tprob == nonlinear_nonstationary_problem && Tp->timecont.tct > 0){ //only for adaptive time controler
      if (ret >= 0) // equilibrium was attained
	{
	  
	  for (k=0;k<n;k++){
	    lhsb[k] = np_gv.lhs[k]; //storing results from previous inner step
	    tdlhsb[k] = np_gv.tdlhs[k]; //storing results from previous inner step
	  }
	  
	  if (ret == 0)
	    nsts++;      
	  dtdef = Tp->timecont.actualforwtimeincr();
	  if (nsts==2)
	    {
	      dt*=2.0;
	      nsts=0;
	      
	      if (Mesprt==1)  
		fprintf (stdout,"\n\n time increment is enlarged because no inner loop was neccessary in previous 3 steps");
	    }
	  if (dt<=dtdef)//increase of time increment according to prescribed one
	    dt = dtdef;
	  
	  if (dt>dtmax)//maximum time increment
	    dt = dtmax;
	}
      else
	{
	  //  reduction of the time increment because
	  //  inner loop was not able to enforce equilibrium
	  dt/=2.0;
	  Tp->timecont.oldtime ();
	  i--;
	  
	  for (k=0;k<n;k++){
	    np_gv.lhs[k] = lhsb[k]; //restoring results from previous time step
	    np_gv.tdlhs[k] = tdlhsb[k]; //restoring results from previous time step
	  }
	  
	  if (Mesprt==1)  
	    fprintf (stdout,"\n\n time increment is reduced because the inner loop was not able to enforce equilibrium");
	  
	  if (dt<dtmin)
	    {
	      if (Mesprt==1)  fprintf (stderr,"\n\n time increment is less than minimum time increment");
	      if (Mesprt==1)  fprintf (stderr,"\n computation failed (file %s, line %d)\n",__FILE__,__LINE__);
	      if (Mesprt==1)  fprintf (stderr,"\n FORCED output of results from this step is performed\n");
	      compute_req_valt (lcid);
	      print_stept(lcid, i, Tp->time, np_gv.rhs);
	      print_flusht();
	      break;
	    }
	}
    }
    
    // adaptivity
    if (Tp->adaptivityflag  &&  Tp->time < end_time)
      if (i%2 == 0)
	breakloop = Adat->run (2, true);
    
  }while(Tp->time < end_time  &&  breakloop == false);
  
  print_closet();
  
  if(Tp->timecont.tct > 0){
    delete [] lhsb;
    delete [] tdlhsb;
  }
}


/**
  Function allocates and initializes global vectors used in npsolver
  It is used in one_step concept solver.

  Created by tomas Krejci according to Tomas Koudelka, 11/2012
*/

void nonstat_solver_init (long lcid, np_glob_vec &np_gv)
{
  long n;
  double dt;

  //  number of unknowns
  n=Ndoft;
  
  np_gv.alloc(n);
  //  nodal values
  np_gv.lhs = Lsrst->give_lhs (lcid);
  //  time derivatives of nodal values
  np_gv.tdlhs = Lsrst->give_tdlhs (lcid);
  //  prescribed fluxes / right hand side
  np_gv.rhs = Lsrst->give_rhs (lcid);
  
  if(Tp->tprob == nonlinear_nonstationary_problem)
    np_gv.alloc_aux(n);//auxiliary vectors

  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();

  ///  termitovo
  if ((Adat != NULL) && (Adat->give_step()>0)) {
    Adat->statedata_restore();
    // timectrl ??
    // i = Tp->istep;
    print_initt(-1, "at");  // ??
  }
  else {
    //  inital printing of output and graphical informations
    if (Tp->hdbcont.restore_stat()){
      solvert_restore (np_gv.lhs, np_gv.tdlhs, np_gv.f, np_gv.istep, Tp->time, dt, Tp->timecont, n);
      //  initiation of transport material models
      Tm->initmaterialmodels();
      compute_req_valt (lcid);
      print_initt(-1, "at");
    }
    else{
      //  initiation of transport material models
      Tm->initmaterialmodels();
      compute_req_valt (lcid);
      print_initt(-1, "wt");
      print_stept(lcid, np_gv.istep, Tp->time, np_gv.rhs);
    }
    print_flusht();
  }
  
  /// termitovo kontrola transferu state variables  DOCASNE
  if (Adat != NULL){
    if (Adat->give_step()) {
      Adat->run (2, true);
      Adat->answer = 0;
    }
  }
}



/**
   function solves linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   according to JK, 20.12.2002,
   revised 05/11/2012 by TKr
*/
long one_step_linear (long lcid,double time, double dt, long istep, long li, np_glob_vec &np_gv)
{
  long j,n;
  double alpha,*f,*d,*p,*lhs,*tdlhs,*rhs;
  
  //new time increment
  Tp->time = time;
  //  new step number
  Tp->istep = istep;
  
  //  nodal values
  lhs = np_gv.lhs;
  //  time derivatives of nodal values
  tdlhs = np_gv.tdlhs;
  //  prescribed fluxes / right hand side
  rhs = np_gv.rhs;
  //  vector of prescribed fluxes (right hand side)
  f = np_gv.f;
  //  predictor
  d = np_gv.d;
  //  auxiliary vector
  p = np_gv.p;
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;

  //  number of transport degrees of freedom
  n=Ndoft;

  if (Mesprt==1){
    fprintf (stdout,"\n\n ------------------------------------------------------------------------");
    fprintf (stdout,"\n TRFEL Time step = %ld,  Time %e,  Time increment = %e",istep,Tp->time,dt);
    fprintf (stdout,"\n ------------------------------------------------------------------------\n");
  }
  
  // update material properties and auxiliary values
  Tm->updateipval ();
  
  //  capacity matrix    
  capacity_matrix (lcid);
  
  //  conductivity matrix
  conductivity_matrix (lcid);
  
  //  predictor
  //  dd = d_n + (1-alpha) dt v_n
  for (j=0;j<n;j++){
    d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
  }
  
  //  auxiliary vector
  //  K dd
  Kmat->gmxv (d,p);
  
  //  matrix of the system of equations
  //  C + K alpha dt
  Kmat->scalgm (dt*alpha);
  Kmat->addgm (1.0,*Cmat);
  
  //  prescribed nodal fluxes f_{n+1}
  trfel_right_hand_side (0,rhs,n);
  
  //  computation of the right hand side vector
  //  f_{n+1} - K dd
  for (j=0;j<n;j++){
    f[j] = rhs[j] - p[j];
  }
  
  //  solution of the system of algebraic equations
  //  (C + alpha dt K) v_{n+1} = f_{n+1} - K dd
  //  time derivatives v_{n+1} are solved
  Tp->ssle->solve_system (Gtt,Kmat,tdlhs,f,Outt);
  
  /*
  fprintf (Outt,"\n\n\n kontrola inverze \n");
  double *jk,*jm;
  jk=new double[n];
  jm=new double[n];
  for (long ikl=0;ikl<n;ikl++){
    for (long ijkl=0;ijkl<n;ijkl++){
      jk[ijkl]=0.0;
    }
    jk[ikl]=1.0;
    Tp->ssle->solve_system (Gtt,Kmat,jm,jk,Outt);
    fprintf (Outt,"\n");
    for (long ijkl=0;ijkl<n;ijkl++){
      fprintf (Outt,"%2ld %2ld  %le\n",ikl,ijkl,jm[ijkl]);
    }
  }
  fprintf (Outt,"\n\n\n konec kontroly inverze \n");
  */
  
  //  nodal values computed from nodal derivatives
  //  d_{n+1} = dd + alpha dt v_{n+1}
  for (j=0;j<n;j++){
    lhs[j] = d[j] + alpha*dt*tdlhs[j];
  }
    
  solution_correction ();
  //  nodes - integration points interpolation
  approximation ();
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();

  compute_req_valt (0);  
  print_stept(0,istep,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
  print_flusht();
  
  if ((Tp->timecont.isitimptime ()==1) && Tp->hdbcont.save_stat())
    solvert_save (lhs,tdlhs,f,istep,Tp->time,dt,Tp->timecont,n);
  
  if (j == 0)
    return 0;
  
  return 1;
}



/**
   function solves non-linear nonstationary transport problem  by Newton-Raphson method
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   according to TKo
   revised 05/11/2012 by TKr
*/
long one_step_nonlinear (long lcid,double time, double dt, long istep, long li, np_glob_vec &np_gv)
{
  long j,k,n,ini;
  double alpha,*f,*d,*p,*lhs,*tdlhs,*rhs;
  double *fb,*fi;
  double norf_last;
  double zero,norf,*err,*thresh;  
  
  //new time increment
  Tp->time = time;
  //  new step number
  Tp->istep = istep;
  
  //  number of transport degrees of freedom
  n=Ndoft;

  //  nodal values
  lhs = np_gv.lhs;
  //  time derivatives of nodal values
  tdlhs = np_gv.tdlhs;
  //  prescribed fluxes / right hand side
  rhs = np_gv.rhs;
  //  vector of prescribed fluxes (right hand side)
  f = np_gv.f;
  //  predictor
  d = np_gv.d;
  //  auxiliary vector
  p = np_gv.p;

  // auxiliary vectors:
  fb = np_gv.fb;
  fi = np_gv.fi;
  
  //  initial values
  nullv (fb,n);
  nullv (fi,n);

  
  //  maximum number of iterations in inner loop
  ini = Tp->nii;
  //  required norm of vector of unbalanced forces
  err = Tp->errarr;
  //  threshold for the size of the right hand side
  thresh=Tp->threshrhs;

  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  //  computer zero
  zero=Tp->zero;

  if (Mesprt==1)  fprintf (stdout,"\n\n --------------------------------------------------------------");
  if (Mesprt==1)  fprintf (stdout,"\n TRFEL Time step = %ld,  Time %e,  Time increment = %e",istep,Tp->time,dt);
  if (Mesprt==1)  fprintf (stdout,"\n --------------------------------------------------------------\n");
  
  //  number of transport degrees of freedom
  n=Ndoft;
  
  // update material properties and auxiliary values
  Tm->updateipval ();
  
  //  capacity matrix    
  capacity_matrix (lcid);
  
  //  conductivity matrix
  conductivity_matrix (lcid);
  
  //  predictor
  //  dd = d_n + (1-alpha) dt v_n
  for (j=0;j<n;j++){
    d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
  }
  
  //  auxiliary vector
  //  K dd
  Kmat->gmxv (d,p);
  
  //  matrix of the system of equations
  //  C + K alpha dt
  Kmat->scalgm (dt*alpha);
  Kmat->addgm (1.0,*Cmat);
  Kmat->copygm (*Cmat);

  //  prescribed nodal fluxes f_{n+1}
  trfel_right_hand_side (0,rhs,n);

  //  computation of the right hand side vector
  //  f_{n+1} - K dd
  for (j=0;j<n;j++){
    f[j] = rhs[j] - p[j];
  }
  //  solution of the system of algebraic equations
  //  (C + alpha dt K) v_{n+1} = f_{n+1} - K dd
  //  time derivatives v_{n+1} are solved
  Tp->ssle->solve_system (Gtt,Cmat,tdlhs,f,Outt);
  
  //  nodal values computed from nodal derivatives
  //  d_{n+1} = dd + alpha dt v_{n+1}
  for (j=0;j<n;j++){
    lhs[j] = d[j] + alpha*dt*tdlhs[j];
  }


  //  physically corrected solution
  solution_correction ();

  //  nodes - integration points interpolation
  approximation ();

  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();

  
  //////////////////////////////////////////////////
  norf_last = 1.0e20;
  //inner iteration loop//////////////////
  for (j=0;j<ini;j++){

    // full newton-raphson
    if (Tp->trsolv == fullnewtont){
      // matrices are computing in each inner iteration
      // capacity matrix
      capacity_matrix (0);
      
      //  conductivity matrix
      conductivity_matrix (0);
      
      //  auxiliary vector  K (d+(1-alpha)*dt*v)
      Kmat->gmxv (d,p);
      
      //  matrix of the system of equations
      //  C + alpha.dt.K
      Kmat->scalgm (dt*alpha);
      Kmat->addgm (1.0,*Cmat);
      Kmat->copygm (*Cmat);
    }
    
    if (Tp->trestype==lrhst){
      //  correct right hand side of transport part
      trfel_right_hand_side (lcid,rhs,n);

      // Solver computes residuum from system of equations
      Kmat->gmxv (tdlhs,fi);//new fi vector
      // Res. = F - K (d+(1-alpha)*dt*v) - (C + alpha*dt*K)*v 
      for (k=0;k<n;k++){
	fb[k] = rhs[k] - p[k] - fi[k];
      }
    }
    
    if (Tp->trestype==fluxest){
      // Solver computes unbalanced fluxes
      internal_fluxes (fi,n);//new fi vector
      //  vector of unbalanced fluxes
      for (k=0;k<n;k++){
	fb[k]=fi[k];
      }
    }
    
    norf = ss (fb,fb,n);
    if (Mesprt==1)  fprintf (stdout,"\n inner iteration %ld   error %e",j,norf);
    if (norf<err[0]) break;
    
    Tp->ssle->solve_system (Gtt,Cmat,fi,fb,Outt);//fi is output now
    
    for (k=0;k<n;k++){
      tdlhs[k]+=fi[k];
      lhs[k]+=alpha*dt*fi[k];
    }
    
    //  physically corrected solution
    solution_correction ();
    //  approximation of nodal values into ontegration points
    approximation ();
    
    // vlozil JM 15.1.2008
    // nulleqother ();
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    //  convergence control for newton-raphson
    //  condition of decreesing of error
    if (Tp->convergcontrolt==yes){
      if (norf > norf_last){
	
	if (Mesprt==1)  fprintf (stdout,"\n\n divergence control: inner iteration skiped %ld error %e\n\n",j,norf);	  
	
	return -2;//break;
      }
      norf_last = norf; //storing norf from previous inner step
    }
    // end of convergence control for newton-raphson

  }
  
  if(norf >= err[0])
    return -1;

  //////////////////////////////////////////////////
  //  printing of output and graphical informations
  compute_req_valt (lcid);
  print_stept(lcid,istep,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
  print_flusht();
  
  if ((Tp->timecont.isitimptime ()==1) && Tp->hdbcont.save_stat())
    solvert_save (lhs,tdlhs,f,istep,Tp->time,dt,Tp->timecont,n);

  return 0;
}


/**
   function solves linear nonstationary transport problem
   the v-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis
   
   JK, 20.12.2002,
   revised 24.8.2006
*/
void linear_nonstat_solv_vform ()
{
  long i,j,n;
  double dt,end_time,alpha,*d,*p,*lhs,*tdlhs,*rhs,*f;
  
  //  number of unknowns
  n=Ndoft;
  
  //  nodal values
  lhs = Lsrst->give_lhs (0);
  nullv (lhs,n);
  //  time derivatives of nodal values
  tdlhs = Lsrst->give_tdlhs (0);
  nullv (tdlhs,n);
  //  prescribed fluxes / right hand side
  rhs = Lsrst->give_rhs (0);
  nullv (rhs,n);
  
  //  vector of prescribed fluxes (right hand side)
  f = new double [n];
  nullv (f,n);
  //  predictor
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  
  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  //  step number
  i=0;
  
  //  initiation of material models
  Tm->initmaterialmodels();
  //  nodes - integration points interpolation
  approximation();  
  
  // pridal Madera 18.1.2008
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();

  ///
  if ((Adat != NULL) && (Adat->give_step()>0)) {
    Adat->statedata_restore();
    // timectrl ??
    i = Tp->istep;
    print_initt(-1, "at");  // ??
  }
  else {
    //  inital printing of output and graphical informations
    if (Tp->hdbcont.restore_stat()){
      solvert_restore (lhs,tdlhs,f,i,Tp->time,dt,Tp->timecont,n);
      print_initt(-1, "at");
      //print_stept(0,i,Tp->time,NULL);
    }
    else{
      compute_req_valt (0);
      print_initt(-1, "wt");
      print_stept(0,i,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
    }
  }
  
  /// termitovo kontrola transferu state variables  DOCASNE
  if (Adat != NULL){
    if (Adat->give_step()) {
      Adat->run (2, true);
      Adat->answer = 0;
    }
  }
  
  bool breakloop = false;
  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    
    //  determination of new time instance
    Tp->time=Tp->timecont.newtime ();
    //  computation of backward time step
    dt = Tp->timecont.actualbacktimeincr ();
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n increment number %ld,   time %f,   time step = %f\n",i,Tp->time,dt);
    
    Tm->updateipval ();
    //  update of step number
    i++;
    Tp->istep = i;
    
    //  capacity matrix    
    capacity_matrix (0);
    

   
    //  conductivity matrix
    conductivity_matrix (0);

    
    //  predictor
    //  dd = d_n + (1-alpha) dt v_n
    for (j=0;j<n;j++){
      d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  auxiliary vector
    //  K dd
    Kmat->gmxv (d,p);
    
    
    //  matrix of the system of equations
    //  C + K alpha dt
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    //  prescribed nodal fluxes f_{n+1}
    trfel_right_hand_side (0,rhs,n);
    
    //  computation of the right hand side vector
    //  f_{n+1} - K dd
    for (j=0;j<n;j++){
      f[j] = rhs[j] - p[j];
    }
    
    //  solution of the system of algebraic equations
    //  (C + alpha dt K) v_{n+1} = f_{n+1} - K dd
    //  time derivatives v_{n+1} are solved
    Tp->ssle->solve_system (Gtt,Kmat,tdlhs,f,Outt);
    
    //  nodal values computed from nodal derivatives
    //  d_{n+1} = dd + alpha dt v_{n+1}
    for (j=0;j<n;j++){

      //debug??!!
      if(lhs[j]>=0.0)
	lhs[j]=-1.0;

      lhs[j] = d[j] + alpha*dt*tdlhs[j];

      //debug??!!
      //if(lhs[j]>=0.0)
      //lhs[j]=-1.0;
    }
    
    solution_correction ();
    //  nodes - integration points interpolation
    approximation ();
    compute_req_valt (0);
    
    // pridal Madera 18.1.2008
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    print_stept(0,i,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
    if ((Tp->timecont.isitimptime ()==1) && Tp->hdbcont.save_stat())
      solvert_save (lhs,tdlhs,f,i,Tp->time,dt,Tp->timecont,n);
    
    /// adaptivity
    if (Tp->adaptivityflag  &&  Tp->time < end_time)
      if (i%2 == 0)
	breakloop = Adat->run (2, true);
    
  }while(Tp->time < end_time  &&  breakloop == false);
  
  delete [] p;
  delete [] d;
  delete [] f;
  
  print_closet();
}



/**
   function solves linear nonstationary transport problem
   the d-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis

   JK, 23.12.2002,
   revised 24.8.2006
*/
void linear_nonstat_solv_dform ()
{
  long i,j,n;
  double dt,end_time,alpha;
  double *d,*f,*p,*lhs,*tdlhs,*rhs,*fi;
  
  //  number of unknowns / degrees of freedom
  n=Ndoft;
  
  //  nodal values
  lhs = Lsrst->give_lhs (0);
  nullv (lhs,n);
  //  time derivatives of nodal values
  tdlhs = Lsrst->give_tdlhs (0);
  nullv (tdlhs,n);
  //  prescribed fluxes / right hand side
  rhs = Lsrst->give_rhs (0);
  nullv (rhs,n);
  
  //  vector of prescribed fluxes (right hand side)
  f = new double [n];
  nullv (f,n);
  //  predictor
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  //  vector of internal fluxes
  fi = new double [n];
  nullv (fi,n);


  //  nodes - integration points interpolation
  approximation ();

  // vlozil JM 15.1.2008
  // nulleqother ();
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  
  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  
  //  step number
  i=0;

  //  initiation of material models
  Tm->initmaterialmodels();
  
  //  inital printing of output and graphical informations
  if (Tp->hdbcont.restore_stat()){
    solvert_restore (lhs,tdlhs,f,i,Tp->time,dt,Tp->timecont,n);
    print_initt(-1, "at");
    //print_stept(0,i,Tp->time,NULL);
  }
  else{
    compute_req_valt (0);
    print_initt(-1, "wt");
    print_stept(0,i,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
  }

  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    
    //  determination of new time instance
    Tp->time=Tp->timecont.newtime ();
    //  computation of backward time step
    dt = Tp->timecont.actualbacktimeincr ();
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n iteration number %ld, time %f, time step = %f\n",i,Tp->time,dt);
    
    Tm->updateipval ();
    //  update of step number
    i++;
    
    //  capacity matrix
    capacity_matrix (0);
    
    /*
    FILE *aux;
    aux = fopen ("matice.txt","a");
    fprintf (aux,"\n\n Matice C ulohy, time %le\n",Tp->time);
    Cmat->printmat (aux);
    */
    
    //  conductivity matrix
    conductivity_matrix (0);
    
    /*
    fprintf (aux,"\n\n Matice K ulohy, time %le\n",Tp->time);
    Kmat->printmat (aux);
    */
    
    //  predictor
    //  dd = d_n + (1-alpha) dt v_n
    for (j=0;j<n;j++){
      d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  auxiliary vector
    //  C dd
    Cmat->gmxv (d,p);
    
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    
    //  prescribed nodal fluxes f_{n+1}
    trfel_right_hand_side (0,rhs,n);

    
    //  computation of the right hand side vector
    //  alpha dt f_n+1 + C dd
    for (j=0;j<n;j++){
      f[j] = rhs[j]*alpha*dt + p[j];
    }
    
    /*
    fprintf (aux,"\n\n Matice C + alpha dt K ulohy, time %le\n",Tp->time);
    Kmat->printmat (aux);
    
    fprintf (aux,"\n\n prava strana ulohy, time %le\n",Tp->time);
    for (j=0;j<n;j++){
      fprintf (aux,"% 15.12le\n",f[j]);
    }
    */
    
    //  solution of the system of algebraic equations
    //  (C + alpha dt K) d_{n+1} = alpha dt f_n+1 + C dd
    //  nodal values d_{n+1} are solved
    Tp->ssle->solve_system (Gtt,Kmat,lhs,f,Outt);
    
    //  first time derivatives of nodal values
    // v_{n+1} = 1/alpha/dt (d_{n+1}-dd)
    for (j=0;j<n;j++){
      tdlhs[j]=(lhs[j]-d[j])/dt/alpha;
    }
    
    /*
    fprintf (aux,"\n\n vektor reseni, time %le\n",Tp->time);
    for (j=0;j<n;j++){
      fprintf (aux,"% 15.12le\n",lhs[j]);
    }
    fclose(aux);
    */

    
    //  computation of values at integration points from nodal values
    solution_correction ();
    approximation ();
    
    // pridal Madera 18.1.2008
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    //  printing of output and graphical informations
    if (Tp->fluxcomp >= 1)
      internal_fluxes (fi,n);

    print_stept(0,i,Tp->time,rhs);// printing of output and graphical informations
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
    if ((Tp->timecont.isitimptime ()==1) && (Tp->hdbcont.hdbtype))
      solvert_save (lhs,tdlhs,f,i,Tp->time,dt,Tp->timecont,n);
    
  }while(Tp->time<end_time);
  
  delete [] fi;
  delete [] f;
  delete [] p;
  delete [] d;
  
  print_closet();
}

/**
   function solves linear nonstationary transport problem
   the d-form version of the generalized trapezoidal method
   see T.J.R. Hughes: The Finite Element Method; Linear Static
   and Dynamic Finite Element Analysis

   JK, 23.12.2002,
   revised 24.8.2006
*/
void linear_nonstat_radiation_solv_dform ()
{
  long i,j,n;
  double dt,end_time,alpha;
  double *d,*f,*p,*lhs,*tdlhs,*rhs,*fi;
  
  //  number of unknowns / degrees of freedom
  n=Ndoft;
  
  //  nodal values
  lhs = Lsrst->give_lhs (0);
  nullv (lhs,n);
  //  time derivatives of nodal values
  tdlhs = Lsrst->give_tdlhs (0);
  nullv (tdlhs,n);
  //  prescribed fluxes / right hand side
  rhs = Lsrst->give_rhs (0);
  nullv (rhs,n);
  
  //  vector of prescribed fluxes (right hand side)
  f = new double [n];
  nullv (f,n);
  //  predictor
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  //  vector of internal fluxes
  fi = new double [n];
  nullv (fi,n);


  //  nodes - integration points interpolation
  approximation ();

  // vlozil JM 15.1.2008
  // nulleqother ();
  if (Tp->nvs==1 && Tp->pnvs==1)
    actual_previous_nodval ();
  
  
  //  coefficient in generalized trapezoidal integration rule
  alpha=Tp->alpha;
  
  //  initial time
  Tp->time=Tp->timecont.starttime ();
  //  end time
  end_time = Tp->timecont.endtime ();
  //  initial time increment
  dt = Tp->timecont.initialtimeincr ();
  
  //  step number
  i=0;

  //  initiation of material models
  Tm->initmaterialmodels();
  
  //  inital printing of output and graphical informations
  if (Tp->hdbcont.restore_stat()){
    solvert_restore (lhs,tdlhs,f,i,Tp->time,dt,Tp->timecont,n);
    print_initt(-1, "at");
    //print_stept(0,i,Tp->time,NULL);
  }
  else{
    compute_req_valt (0);
    print_initt(-1, "wt");
    print_stept(0,i,Tp->time,rhs);//print_stept(0,i,Tp->time,NULL);
  }
  
  // ***************************
  //  main iteration loop  ****
  // ***************************
  do{
    
    //  determination of new time instance
    Tp->time=Tp->timecont.newtime ();
    //  computation of backward time step
    dt = Tp->timecont.actualbacktimeincr ();
    
    if (Mesprt != 0)  fprintf (stdout,"\n\n iteration number %ld, time %f, time step = %f\n",i,Tp->time,dt);
    
    Tm->updateipval ();
    //  update of step number
    i++;
        
    //  capacity matrix
    capacity_matrix (0);
    

    //  conductivity matrix
    conductivity_matrix (0);

    //fprintf (Outt,"\n\n matice vodivosti \n");
    //Kmat->printmat (Outt);
    //fprintf (Outt,"\n\n konec matice vodivosti \n\n\n");
    //fprintf (Outt,"\n\n matice kapacity \n");
    //Cmat->printmat (Outt);
    //fprintf (Outt,"\n\n konec matice kapacity \n\n\n");
    
    //  predictor
    //  dd = d_n + (1-alpha) dt v_n
    for (j=0;j<n;j++){
      d[j]=lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  auxiliary vector
    //  C dd
    Cmat->gmxv (d,p);
    
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    Kmat->scalgm (dt*alpha);
    Kmat->addgm (1.0,*Cmat);
    
    
    //  prescribed nodal fluxes f_{n+1}
    trfel_right_hand_side (0,rhs,n);

    
    //  computation of the right hand side vector
    //  alpha dt f_n+1 + C dd
    for (j=0;j<n;j++){
      f[j] = rhs[j]*alpha*dt + p[j];
    }
    
    Tt->edge_temperature ();
    Tt->heat_fluxes (f,Outt);

    //fprintf (Outt,"\n\n matice ulohy \n");
    //Kmat->printmat (Outt);
    //fprintf (Outt,"\n\n konec matice ulohy \n\n\n");
    //fprintf (Outt,"\n\n vektor prave strany \n");
    //for (j=0;j<n;j++){
    //fprintf (Outt,"\n f %le",f[j]);
    //}
    //fprintf (Outt,"\n\n konec vektoru prave strany \n\n\n");

    //  solution of the system of algebraic equations
    //  (C + alpha dt K) d_{n+1} = alpha dt f_n+1 + C dd
    //  nodal values d_{n+1} are solved
    Tp->ssle->solve_system (Gtt,Kmat,lhs,f,Outt);
    
    //  first time derivatives of nodal values
    // v_{n+1} = 1/alpha/dt (d_{n+1}-dd)
    for (j=0;j<n;j++){
      tdlhs[j]=(lhs[j]-d[j])/dt/alpha;
    }
    
    //  computation of values at integration points from nodal values
    solution_correction ();
    approximation ();
    
    // pridal Madera 18.1.2008
    if (Tp->nvs==1 && Tp->pnvs==1)
      actual_previous_nodval ();
    
    //  printing of output and graphical informations
    if (Tp->fluxcomp >= 1)
      internal_fluxes (fi,n);

    print_stept(0,i,Tp->time,rhs);// printing of output and graphical informations
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
    
    if ((Tp->timecont.isitimptime ()==1) && (Tp->hdbcont.hdbtype))
      solvert_save (lhs,tdlhs,f,i,Tp->time,dt,Tp->timecont,n);
    
  }while(Tp->time<end_time);
  
  delete [] fi;
  delete [] f;
  delete [] p;
  delete [] d;
  
  print_closet();
}

